@@ -390,11 +390,10 @@ Application::Application ():
 	}
 	
 #ifdef HAVE_GO_CONF_SYNC
-		m_ConfNode = go_conf_get_node (GetConfDir (), GCP_CONF_DIR_SETTINGS);
+	m_ConfNode = go_conf_get_node (GetConfDir (), GCP_CONF_DIR_SETTINGS);
 #else
-		GError *error = NULL;
-		m_ConfClient = gconf_client_get_default ();
-		gconf_client_add_dir (m_ConfClient, "/apps/gchemutils/paint/settings", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+	GError *error = NULL;
+	m_ConfClient = gconf_client_get_default ();
 #endif
 	GCU_GCONF_GET ("compression", int, CompressionLevel, 0)
 	GCU_GCONF_GET ("tearable-mendeleiev", bool, TearableMendeleiev, false)
@@ -412,7 +411,7 @@ Application::Application ():
 	Theme *pTheme;
 	m_Dummy = new Object (0);
 	for (j = Names.begin (); j != jend; j++) {
-		pTheme = TheThemeManager.GetTheme ("Default");
+		pTheme = TheThemeManager.GetTheme (*j);
 		pTheme->AddClient (m_Dummy);
 	}
 }
@@ -426,6 +425,14 @@ Application::~Application ()
 	if (XmlDoc)
 		xmlFreeDoc (XmlDoc);
 	m_SupportedMimeTypes.clear ();
+	// remove themes permanency with this as a dummy client
+	list <string> Names = TheThemeManager.GetThemesNames ();
+	list <string>::iterator j, jend = Names.end ();
+	Theme *pTheme;
+	for (j = Names.begin (); j != jend; j++) {
+		pTheme = TheThemeManager.GetTheme (*j);
+		pTheme->RemoveClient (m_Dummy);
+	}
 	delete m_Dummy;
 #ifdef HAVE_GO_CONF_SYNC
 	go_conf_remove_monitor (m_NotificationId);
@@ -433,7 +440,6 @@ Application::~Application ()
 	m_ConfNode = NULL;
 #else
 	gconf_client_notify_remove (m_ConfClient, m_NotificationId);
-	gconf_client_remove_dir (m_ConfClient, "/apps/gchemutils/gl", NULL);
 	g_object_unref (m_ConfClient);
 	m_ConfClient = NULL;
 #endif
@@ -488,6 +494,8 @@ enum {
 	CHEMISTRY,
 	SVG,
 	EPS,
+	PDF,
+	PS,
 	PIXBUF
 };
 
@@ -511,6 +519,10 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
 	if (filename[i] == '/') i = 0;
 	ext = (i > 0)? filename + i + 1: NULL;
+	if (!mime_type) // probably a non existing file
+		mime_type = gnome_vfs_get_file_mime_type_fast (filename, NULL);
+	if (!mime_type) // to be really sure we don't crash
+		mime_type = "application/x-gchempaint";
 	list<string>::iterator it, itend = m_SupportedMimeTypes.end ();
 	for (it =  m_SupportedMimeTypes.begin (); it != itend; it++)
 		if (*it == mime_type) {
@@ -522,6 +534,10 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 			file_type = SVG;
 		else if (!strcmp (mime_type, "image/x-eps"))
 			file_type = EPS;
+		else if (!strcmp (mime_type, "application/pdf"))
+			file_type = PDF;
+		else if (!strcmp (mime_type, "application/ps"))
+			file_type = PS;
 		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type))) {
 			file_type = PIXBUF;
 			if (!ext) {
@@ -581,6 +597,12 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 			case EPS:
 				m_pActiveDoc->ExportImage (filename2, "eps");
 				break;
+			case PDF:
+				m_pActiveDoc->ExportImage (filename2, "pdf");
+				break;
+			case PS:
+				m_pActiveDoc->ExportImage (filename2, "ps");
+				break;
 			case PIXBUF:
 				m_pActiveDoc->ExportImage (filename2, pixbuf_type, GetImageResolution ());
 				break;
@@ -622,6 +644,9 @@ bool Application::FileProcess (const gchar* filename, const gchar* mime_type, bo
 		if (Load (filename2, mime_type, pDoc)) {
 			pDoc->GetView ()->AddObject (pDoc);
 			pDoc->GetView ()->Update (pDoc);
+			pDoc->GetView ()->EnsureSize ();
+			if (pDoc->GetWindow ())
+				pDoc->GetWindow ()->ActivateActionWidget ("/MainMenu/FileMenu/SaveAsImage", pDoc->HasChildren ());
 		} else {
 			if (create) {
 				pDoc->GetWindow ()->Destroy ();;
@@ -673,7 +698,7 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 		if (local) {
 			ifstream ifs;
 			GnomeVFSURI *uri = gnome_vfs_uri_new (filename.c_str ());
-			ifs.open (gnome_vfs_uri_get_path (uri));
+			ifs.open (g_uri_unescape_string (gnome_vfs_uri_get_path (uri),NULL));
 			gnome_vfs_uri_unref (uri);
 			if (ifs.fail ())
 				throw (int) 1;
@@ -774,22 +799,21 @@ void Application::OpenWithBabel (string const &filename, const gchar *mime_type,
 		switch (num)
 		{
 		case 0:
-			mess = g_strdup_printf(_("No filename"));
+			mess = _("No filename");
 			break;
 		case 1:
-			mess = g_strdup_printf(_("Could not open file\n%s"), filename.c_str ());
+			mess = _("Could not open file\n%s");
 			break;
 		case 2:
-			mess = g_strdup_printf(_("%s: parse error."), filename.c_str ());
+			mess = _("%s: parse error.");
 			break;
 		default:
 			throw (num); //this should not occur
 		}
-		message = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+		message = gtk_message_dialog_new (NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, g_uri_unescape_string (filename.c_str (), NULL));
 		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
 		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
 		gtk_widget_show(message);
-		g_free(mess);
 	}
 }
 
@@ -887,28 +911,27 @@ void Application::OpenGcp (string const &filename, Document* pDoc)
 		switch (num)
 		{
 		case 0:
-			mess = g_strdup_printf(_("No filename"));
+			mess = _("No filename");
 			break;
 		case 1:
-			mess = g_strdup_printf(_("Could not load file\n%s"), filename.c_str ());
+			mess = _("Could not load file\n%s");
 			break;
 		case 2:
-			mess = g_strdup_printf(_("%s: invalid xml file.\nTree is empty?"), filename.c_str ());
+			mess = _("%s: invalid xml file.\nTree is empty?");
 			break;
 		case 3:
-			mess = g_strdup_printf(_("%s: invalid file format."), filename.c_str ());
+			mess = _("%s: invalid file format.");
 			break;
 		case 4:
-			mess = g_strdup_printf(_("%s: parse error."), filename.c_str ());
+			mess = _("%s: parse error.");
 			break;
 		default:
 			throw (num); //this should not occur
 		}
-		message = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+		message = gtk_message_dialog_new(NULL, (GtkDialogFlags) 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess, g_uri_unescape_string (filename.c_str (), NULL));
 		gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
 		g_signal_connect_swapped (G_OBJECT (message), "response", G_CALLBACK (gtk_widget_destroy), G_OBJECT (message));
 		gtk_widget_show(message);
-		g_free(mess);
 	}
 }
 
@@ -928,6 +951,8 @@ void Application::OnSaveAsImage ()
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
 		l.push_front ((*i).first.c_str ());
+	l.push_front ("application/ps");
+	l.push_front ("application/pdf");
 	l.push_front ("image/x-eps");
 	l.push_front ("image/svg+xml");
 	FileChooser (this, true, l, m_pActiveDoc, _("Save as image"), GetImageResolutionWidget ());
