@@ -4,7 +4,7 @@
  * GChemPaint library
  * arrow.h 
  *
- * Copyright (C) 2002-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -42,16 +42,17 @@ public:
 	Arrow(gcu::TypeId Type);
 	virtual ~Arrow();
 	
-	virtual bool Load (xmlNodePtr);
-	virtual void SetSelected (GtkWidget* w, int state);
+	bool Load (xmlNodePtr);
+	void SetSelected (GtkWidget* w, int state);
 	void SetCoords (double xstart, double ystart, double xend, double yend);
 	bool GetCoords (double* xstart, double* ystart, double* xend, double* yend);
-	virtual void Move (double x, double y, double z = 0);
-	virtual void Transform2D (gcu::Matrix2D& m, double x, double y);
-	virtual double GetYAlign ();
+	void Move (double x, double y, double z = 0);
+	void Transform2D (gcu::Matrix2D& m, double x, double y);
+	double GetYAlign ();
+	bool SetProperty (unsigned property, char const *value);
 
 protected:
-	bool Save (xmlDocPtr xml, xmlNodePtr node);
+	bool Save (xmlDocPtr xml, xmlNodePtr node) const;
 	
 protected:
 	double m_x, m_y, m_width, m_height;
