@@ -4,7 +4,7 @@
  * GChemPaint library
  * fragment-atom.cc 
  *
- * Copyright (C) 2003-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2003-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,6 +26,7 @@
 #include "fragment-atom.h"
 #include "fragment.h"
 #include "molecule.h"
+#include "view.h"
 #include <gcu/element.h>
 #include <cstring>
 
@@ -43,6 +44,7 @@ FragmentAtom::FragmentAtom (Fragment *fragment, int Z): Atom ()
 	m_Fragment = fragment;
 	SetZ (Z);
 	SetId ("a1");
+	BuildSymbolGeometry (0,0,0);
 }
 
 FragmentAtom::~FragmentAtom () {}
@@ -57,6 +59,7 @@ void FragmentAtom::SetZ (int Z)
 	if (Z != 0)
 		m_Fragment->OnChangeAtom ();
 	setting = false;
+	BuildSymbolGeometry (0,0,0);
 }
 
 /*!
@@ -70,14 +73,14 @@ bool FragmentAtom::AcceptNewBonds (int nb)
 /*!
 Overrided to avoid Atom::Add execution. Don't do anything.
 */
-void FragmentAtom::Add (GtkWidget* w)
+void FragmentAtom::Add (GtkWidget* w) const
 {
 }
 
 /*!
 Overrided to avoid Atom::Update execution. Just call fragment Update method.
 */
-void FragmentAtom::Update (GtkWidget* w)
+void FragmentAtom::Update (GtkWidget* w) const
 {
 	m_Fragment->Update (w);
 }
@@ -91,7 +94,7 @@ void FragmentAtom::SetSelected (GtkWidget* w, int state)
 	m_Fragment->SetSelected (w, state);
 }
 
-xmlNodePtr FragmentAtom::Save (xmlDocPtr xml)
+xmlNodePtr FragmentAtom::Save (xmlDocPtr xml) const
 {
 	xmlNodePtr node;
 	gchar buf[16];
@@ -251,6 +254,37 @@ bool FragmentAtom::AcceptCharge (int charge)
 
 void FragmentAtom::Update ()
 {
+	m_Fragment->Update ();
+}
+
+bool FragmentAtom::Match (gcu::Atom *atom, AtomMatchState &state)
+{
+	return false; // not supported at the moment
+}
+
+void FragmentAtom::DoBuildSymbolGeometry (View *pView)
+{
+	// Building atom geometry if necessary
+	double ascent, x;
+	GetSymbolGeometry (ascent, x, x, false);
+	if (ascent == 0.) {
+		PangoContext* pc = pView->GetPangoContext ();
+		PangoLayout *layout = pango_layout_new (pc);
+		pango_layout_set_font_description (layout, pView->GetPangoFontDesc ());
+		PangoRectangle rect;
+		if (m_CHeight == 0.) {
+			pango_layout_set_text (layout, "C", 1);
+			pango_layout_get_extents (layout, &rect, NULL);
+			m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
+		}
+		pango_layout_set_text (layout, GetSymbol (), -1);
+		PangoLayoutIter* iter = pango_layout_get_iter (layout);
+		ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
+		pango_layout_get_extents (layout, &rect, NULL);
+		BuildSymbolGeometry ((double) rect.width / PANGO_SCALE, (double) rect.height / PANGO_SCALE, ascent - (double) rect.y / PANGO_SCALE - m_CHeight);
+		g_object_unref (G_OBJECT (layout));
+	}
 }
 
 }	//	namespace gcp
