@@ -99,6 +99,19 @@ Molecule::~Molecule ()
 {
 }
 
+void Molecule::Add (GtkWidget* w) const
+{
+	std::list<gcu::Atom*>::const_iterator i, iend = m_Atoms.end ();
+	for (i = m_Atoms.begin (); i != iend; i++)
+		(*i)->Add (w);
+	std::list<Fragment*>::const_iterator j, jend = m_Fragments.end ();
+	for (j = m_Fragments.begin (); j != jend; j++)
+		(*j)->Add (w);
+	std::list<gcu::Bond*>::const_iterator k, kend = m_Bonds.end ();
+	for (k = m_Bonds.begin (); k != kend; k++)
+		(*k)->Add (w);
+}
+
 void Molecule::AddChild (Object* object)
 {
 	switch (object->GetType ()) {
@@ -582,7 +595,7 @@ void Molecule::SelectAlignmentItem (Object *child)
 	EmitSignal (OnChangedSignal);
 }
 
-xmlNodePtr Molecule::Save (xmlDocPtr xml)
+xmlNodePtr Molecule::Save (xmlDocPtr xml) const
 {
 	xmlNodePtr node = Object::Save (xml);
 	if (!node)
@@ -836,14 +849,14 @@ char const *Molecule::GetInChI ()
 	return m_InChI.c_str ();
 }
 
-std::string Molecule::GetRawFormula ()
+std::string Molecule::GetRawFormula () const
 {
 	ostringstream ofs;
 
 	if (!m_Fragments.size ()) {
 		// we do not support fragments at the moment
 		map<string, int> elts;
-		list<gcu::Atom*>::iterator ia, enda = m_Atoms.end ();
+		list<gcu::Atom*>::const_iterator ia, enda = m_Atoms.end ();
 		for (ia = m_Atoms.begin(); ia != enda; ia++) {
 			if ((*ia)->GetZ () == 0)
 				continue;
@@ -871,20 +884,9 @@ void Molecule::OnLoaded ()
 	UpdateCycles ();
 }
 
-bool Molecule::operator== (Molecule const& molecule)
+unsigned Molecule::GetAtomsNumber () const
 {
-	// first examine each atom of each molecule and sort by Z.
-	map<int, set<gcu::Atom*> > atoms1, atoms2;
-	list<gcu::Atom*>::const_iterator ia, enda = m_Atoms.end ();
-	for (ia = m_Atoms.begin ();  ia != enda; ia++)
-		atoms1[(*ia)->GetZ ()].insert (*ia);
-	enda = molecule.m_Atoms.end ();
-	for (ia = molecule.m_Atoms.begin ();  ia != enda; ia++)
-		atoms2[(*ia)->GetZ ()].insert (*ia);
-	if (atoms1.size () != atoms2.size ())
-		return false;
-	// TODO: write this code
-	return true;
+	return m_Atoms.size () + m_Fragments.size ();
 }
 
 }	//	namespace gcp
