@@ -4,7 +4,7 @@
  * GChemPaint library
  * stringdlg.cc
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,10 +26,12 @@
 #include "stringdlg.h"
 #include "document.h"
 #include "application.h"
+#include "widgetdata.h"
 #include "window.h"
 #include <libgnomevfs/gnome-vfs-ops.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <glib/gi18n-lib.h>
+#include <sstream>
 #include <cstring>
 
 using namespace std;
