@@ -35,9 +35,12 @@
 #include "window.h"
 #include <canvas/gprintable.h>
 #include <canvas/gcp-canvas-group.h>
+#include <canvas/gcp-canvas-rect-ellipse.h>
+#include <gsf/gsf-output-gio.h>
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
-#include <unistd.h>
+#include <cairo-pdf.h>
+#include <cairo-ps.h>
 #include <pango/pango-context.h>
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n-lib.h>
@@ -48,6 +51,7 @@
 #include <string>
 #include <cstring>
 #include <iostream>
+#include <unistd.h>
 
 /*
 Derivation of a new widget from gnome_canvas with an event for updating canvas size
@@ -138,8 +142,10 @@ void gnome_canvas_gcp_init (GnomeCanvasGCP *canvas)
 
 void gnome_canvas_gcp_update_bounds (GnomeCanvasGCP *canvas)
 {
-	while (gtk_events_pending ())
-		gtk_main_iteration ();
+	GnomeCanvas* w = GNOME_CANVAS (canvas);
+	while (w->idle_id)
+		gtk_main_iteration();
+	gnome_canvas_update_now (w);
 	gcp::WidgetData* pData = (gcp::WidgetData*) g_object_get_data (G_OBJECT (canvas), "data");
 	double x1, y1, x2, y2;
 	gnome_canvas_item_get_bounds (GNOME_CANVAS_ITEM (pData->Group), &x1, &y1, &x2, &y2);
@@ -241,7 +247,7 @@ bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 	if (event->type == GDK_BUTTON_PRESS) {
 		if (item == m_pData->Background) {
 			item = NULL;
-			std::map<Object*, GnomeCanvasGroup*>::iterator i = m_pData->Items.begin (),
+			std::map<Object const*, GnomeCanvasGroup*>::iterator i = m_pData->Items.begin (),
 						end = m_pData->Items.end ();
 			Bond* pBond;
 			while (i != end) {
@@ -318,7 +324,7 @@ bool View::OnEvent (GnomeCanvasItem *item, GdkEvent *event, GtkWidget* widget)
 	return false;
 }
 
-void View::AddObject (Object* pObject)
+void View::AddObject (Object const *pObject)
 {
 	std::list<GtkWidget*>::iterator i, end = m_Widgets.end ();
 	for (i = m_Widgets.begin (); i != end; i++)
@@ -343,7 +349,7 @@ GtkWidget* View::CreateNewWidget ()
 		m_pData->Zoom = 1.0;
 		m_pData->Background = gnome_canvas_item_new (
 									gnome_canvas_root (GNOME_CANVAS (m_pWidget)),
-									gnome_canvas_rect_get_type (),
+									gnome_canvas_rect_ext_get_type (),
 									"x1", 0.0,
 									"y1", 0.0,
 									"x2", (double) m_width,
@@ -363,7 +369,7 @@ GtkWidget* View::CreateNewWidget ()
 		m_Widgets.push_back (m_pWidget);
 		if (pWidget) {
 			WidgetData* pData = (WidgetData*) g_object_get_data (G_OBJECT (pWidget), "data");
-			std::map<Object*, GnomeCanvasGroup*>::iterator i, iend = pData->Items.end ();
+			std::map<Object const*, GnomeCanvasGroup*>::iterator i, iend = pData->Items.end ();
 			for (i = pData->Items.begin (); i != iend; i++)
 				if ((*i).first->GetType () != gcu::BondType)
 				 (*i).first->Add (m_pWidget);
@@ -399,32 +405,7 @@ GnomeCanvasItem* View::GetCanvasItem (GtkWidget* widget, Object* Object)
 	return result;
 }
 
-/*void View::Print (GnomePrintContext *pc, gdouble width, gdouble height)
-{
-	g_return_if_fail (G_IS_PRINTABLE (m_pData->Group));
-	gnome_print_gsave (pc);
-	double matrix [6] = {.75, 0., 0., -.75, 0., height};
-	if (!m_bEmbedded) {
-		//FIXME: add a print setup dialog and use real margins values
-		matrix[4] += 30;
-		matrix[5] -= 30;
-	}
-	m_pData->ShowSelection(false);
-	Object* pObj = NULL;
-	if (m_ActiveRichText) {
-		pObj = (Object*) g_object_get_data (G_OBJECT (m_ActiveRichText), "object");
-		if (pObj) pObj->SetSelected (m_pWidget, SelStateUnselected);
-	}
-	gnome_print_concat (pc, matrix);
-	GPrintable* printable = G_PRINTABLE (m_pData->Group);
-	(* G_PRINTABLE_GET_IFACE (printable)->print) (G_PRINTABLE (printable), pc); 
-	gnome_print_grestore (pc);
-	m_pData->ShowSelection (true);
-	if (pObj)
-		pObj->SetSelected (m_pWidget, SelStateUpdating);
-}*/
-
-void View::Update (Object* pObject)
+void View::Update (Object const *pObject)
 {
 	std::list<GtkWidget*>::iterator i;
 	for (i = m_Widgets.begin (); i != m_Widgets.end (); i++)
@@ -534,14 +515,15 @@ void View::OnDeleteSelection (GtkWidget* w)
 }
 
 GtkTargetEntry const targets[] = {
-	{(char *) GCHEMPAINT_ATOM_NAME,  0, 0},
-	{(char *) "image/svg",  0, 1},
-	{(char *) "image/svg+xml",  0, 2},
-	{(char *) "image/png",  0, 3},
-	{(char *) "image/jpeg",  0, 4},
-	{(char *) "image/bmp",  0, 5},
-	{(char *) "UTF8_STRING", 0, 6},
-	{(char *) "STRING", 0, 7}
+	{(char *) GCHEMPAINT_ATOM_NAME,  0, GCP_CLIPBOARD_NATIVE},
+	{(char *) "image/svg",  0, GCP_CLIPBOARD_SVG},
+	{(char *) "image/svg+xml",  0, GCP_CLIPBOARD_SVG_XML},
+	{(char *) "image/x-eps",  0, GCP_CLIPBOARD_EPS},
+	{(char *) "image/png",  0, GCP_CLIPBOARD_PNG},
+	{(char *) "image/jpeg",  0, GCP_CLIPBOARD_JPEG},
+	{(char *) "image/bmp",  0, GCP_CLIPBOARD_BMP},
+	{(char *) "UTF8_STRING", 0, GCP_CLIPBOARD_UTF8_STRING},
+	{(char *) "STRING", 0, GCP_CLIPBOARD_STRING}
 };
 
 void View::OnCopySelection (GtkWidget* w, GtkClipboard* clipboard)
@@ -975,187 +957,49 @@ static gboolean do_save_image (const gchar *buf, gsize count, GError **error, gp
 	return true;
 }
 
+static cairo_status_t cairo_write_func (void *closure, const unsigned char *data, unsigned int length)
+{
+	gboolean result;
+	GsfOutput *output = GSF_OUTPUT (closure);
+
+	result = gsf_output_write (output, length, data);
+
+	return result ? CAIRO_STATUS_SUCCESS : CAIRO_STATUS_WRITE_ERROR;
+}
+
 void View::ExportImage (string const &filename, const char* type, int resolution)
 {
 	ArtDRect rect;
 	m_pData->GetObjectBounds (m_pDoc, &rect);
 	m_pData->ShowSelection (false);
 	int w = (int) (ceil (rect.x1) - floor (rect.x0)), h = (int) (ceil (rect.y1) - floor (rect.y0));
-	if (!strcmp (type, "eps")) {
-/*		GnomePrintConfig* config = gnome_print_config_default();
-		GnomePrintContext *pc;
-		GnomePrintJob *gpj = gnome_print_job_new (config);
-		pc = gnome_print_job_get_context (gpj);
-		gnome_print_beginpage(pc, (const guchar*)"");
-		gdouble width, height;
-		gnome_print_config_get_page_size(config, &width, &height);
-		Print(pc, width, height);
-		gnome_print_showpage(pc);
-		g_object_unref(pc);
-		gnome_print_job_close(gpj);
-		char *tmpname = g_strdup ("/tmp/2epsXXXXXX");
-		int f = g_mkstemp (tmpname);
-		close (f);
-		double hp, mt, ml;
-		GnomePrintUnit const *unit;
-		GnomePrintUnit const *inches = gnome_print_unit_get_by_abbreviation ((const guchar *) "in");
-		gnome_print_config_get_length (config, (const guchar*) "Settings.Output.Media.PhysicalSize.Height", &hp, &unit);
-		gnome_print_convert_distance (&hp, unit, inches);
-		hp *= 72;*/
-/*		gnome_print_config_get_length (config, (const guchar*) "Settings.Document.Page.Margins.Left", &ml, &unit);
-		gnome_print_convert_distance (&ml, unit, inches);
-		ml *= 72;
-		gnome_print_config_get_length (config, (const guchar*) "Settings.Document.Page.Margins.Top", &mt, &unit);
-		gnome_print_convert_distance (&mt, unit, inches);
-		mt *= 72;	*/
-/*		ml = mt = 30.; // see View::Print !
-		gnome_print_config_set_boolean (config, (const guchar*) "Settings.Output.Job.PrintToFile", true);
-		gnome_print_config_set (config, (const guchar*) GNOME_PRINT_KEY_OUTPUT_FILENAME, (const guchar*) tmpname);
-		gnome_print_job_print (gpj);
-		g_object_unref (gpj);
-		gnome_print_config_unref(config);
-		char buf[256];
-		ifstream *fin = new ifstream (tmpname);
-		ostringstream fout (filename.c_str ());
-		fout << "%!PS-Adobe-3.0 EPSF-3.0" << endl;
-		fout << "%%BoundingBox: " << (int) (rect.x0 * .75 + ml) << " " << (int) (hp - mt - rect.y1 * .75)  <<  " "  << (int) (rect.x1 * .75 + ml) << " " << (int) (hp - mt - rect.y0 * .75) << endl;
-		fout << "%%HiResBoundingBox: " << rect.x0 * .75 + ml << " " << hp - mt - rect.y1 * .75 <<  " "  << rect.x1 * .75 + ml << " " << hp - mt - rect.y0 * .75 << endl;
-		fin->getline (buf, 256);
-		while (!fin->eof ()) {
-			fin->getline (buf, 256);
-			if (strlen (buf) >= 255)
-				exit (-1); *//* This is violent but should not occur */
-/*			if (!strncmp (buf, "%%", 2)) {
-				if (!strncmp (buf + 2, "Orientation", strlen ("Orientation")))
-					continue;
-				else if (!strncmp (buf + 2, "Pages", strlen ("Pages")))
-					continue;
-				else if (!strncmp (buf + 2, "BoundingBox", strlen ("BoundingBox")))
-					continue;
-				else if (!strncmp (buf + 2, "BeginDefaults", strlen ("BeginDefaults")))
-					continue;
-				else if (!strncmp (buf + 2, "EndDefaults", strlen ("EndDefaults")))
-					continue;
-				else if (!strncmp (buf + 2, "BeginSetup", strlen ("BeginSetup")))
-					continue;
-				else if (!strncmp (buf + 2, "EndSetup", strlen ("EndSetup")))
-					continue;
-				else if (!strncmp (buf + 2, "PageMedia", strlen ("PageMedia")))
-					continue;
-				else if (!strncmp (buf + 2, "EndProlog", strlen ("EndProlog")))
-					continue;
-				else if (!strncmp (buf + 2, "BeginResource", strlen ("BeginResource")))
-					continue;
-				else if (!strncmp (buf + 2, "EndResource", strlen ("EndResource")))
-					continue;
-				else if (!strncmp (buf + 2, "Page: ", strlen ("Page: ")))
-					continue;
-				else if (!strncmp (buf + 2, "%%", 2))
-					continue;
-				else if (!strncmp (buf + 2, "PageResources", strlen ("PageResources")))
-					continue;
-				else if (!strncmp (buf + 2, "EndComments", strlen ("EndComments"))) {
-					 fout << buf << endl;
-					if (!filename.compare (filename.length () - 5, 5, ".epsi")) {
-						gnome_canvas_update_now (GNOME_CANVAS (m_pWidget));
-						GdkPixbuf *pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, w, h);
-						GnomeCanvasBuf cbuf;
-						int i, j, mask, lines, bytes;
-						cbuf.buf = gdk_pixbuf_get_pixels (pixbuf);
-						cbuf.rect.x0 = (int) floor (rect.x0);
-						cbuf.rect.x1 = (int) ceil (rect.x1);
-						cbuf.rect.y0 = (int) floor (rect.y0);
-						cbuf.rect.y1 = (int) ceil (rect.y1);
-						cbuf.buf_rowstride = gdk_pixbuf_get_rowstride (pixbuf);
-						cbuf.bg_color = 0xffffff;
-						cbuf.is_buf = 1;
-						(* GNOME_CANVAS_ITEM_GET_CLASS (m_pData->Group)->render) (GNOME_CANVAS_ITEM (m_pData->Group), &cbuf);
-	*/					/* use 8 bits depth, no more than 250 bytes per line*/
-	/*					lines = (w + 249) / 250 * h;
-						fout << "%%BeginPreview: " << w << " " << h << " " << 8 << " " << lines << endl;
-						fout << hex;
-						for (j = 0; j < h; j++) {
-							bytes = 0;
-							for (i = 0; i < 3 * w; i+= 3) {
-								if (bytes == 0)
-									fout << "%";
-								mask = 0xff - (cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3;
-								fout << (mask & 0xf);
-								mask = mask >> 4;
-								fout << mask;
-								bytes++;
-								if (bytes == 250) {
-									fout << endl;
-									bytes = 0;
-								}
-							}
-							if (bytes)
-								fout << endl;
-							cbuf.buf += cbuf.buf_rowstride;
-						}*/
-						/* use 1 bit depth, no more than 128 bytes per line (max width 1024) */
-/*						lines = (w + 1023) / 1024 * h;
-						fout << "%%BeginPreview: " << w << " " << h << " " << 1 << " " << lines << endl;
-						fout << hex;
-						for (j = 0; j < h; j++) {
-							mask = 1; b = 0;
-							bytes = 0;
-							for (i = 0; i < 3 * w; i+= 3) {
-								if (bytes == 0)
-									fout << "%";
-								if ((cbuf.buf[i] + cbuf.buf[i + 1] + cbuf.buf[i + 2]) / 3 < 0x80)
-									b |= mask;
-								mask <<= 1;
-								if (mask == 16) {
-									fout << (int) b;
-									mask = 1;
-									b = 0;
-									bytes++;
-									if (bytes == 128) {
-										fout << endl;
-										bytes = 0;
-									}
-								}
-							}
-							if (mask != 1) {
-								fout << (int) b;
-								bytes++;
-							}
-							if (bytes)
-								fout << endl;
-							cbuf.buf += cbuf.buf_rowstride;
-						}*/
-/*						fout << dec;
-						fout << "%%EndPreview" << endl;
-						g_object_unref (pixbuf);
-					}
-				} else if (!strncmp (buf + 2, "BeginProlog", strlen ("BeginProlog"))) {
-					 fout << buf << endl;
-					 fout << "save" << endl;
-					 fout << "countdictstack" << endl;
-					 fout << "mark" << endl;
-					 fout << "newpath" << endl;
-					 fout << "/showpage {} def" << endl;
-					 fout << "/setpagedevice {pop} def" << endl;
-					 fout << "%%EndProlog" << endl;
-					 fout << "%%Page 1 1" << endl;
-				} else if (!strncmp (buf + 2, "Trailer", strlen ("Trailer"))) {
-					 fout << buf << endl;
-					 fout << "cleartomark" << endl;
-					 fout << "countdictstack" << endl;
-					 fout << "exch sub { end } repeat" << endl;
-					 fout << "restore" << endl;
-				} else 
-					 fout << buf << endl;
-			} else
-			 fout << buf << endl;
+	if (!strcmp (type, "eps") || !strcmp (type, "ps") || !strcmp (type, "pdf")) {
+		GError *error = NULL;
+		GsfOutput *output = gsf_output_gio_new_for_uri (filename.c_str (), &error);
+		if (error) {
+			gchar * mess = g_strdup_printf (_("Could not create stream!\n%s"), error->message);
+			GtkWidget* message = gtk_message_dialog_new (GTK_WINDOW (gtk_widget_get_toplevel (m_pWidget)), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+			g_free (mess);
+			gtk_dialog_run (GTK_DIALOG (message));
+			gtk_widget_destroy (message);
+			g_error_free (error);
 		}
-		fin->close ();
-		delete fin;
-		GnomeVFSHandle *handle = NULL;
-		GnomeVFSFileSize n;
-		if (gnome_vfs_create (&handle, filename.c_str (), GNOME_VFS_OPEN_WRITE, true, 0644) == GNOME_VFS_OK)
-			gnome_vfs_write (handle, fout.str ().c_str (), (GnomeVFSFileSize) fout.str ().size (), &n);*/
+		ArtDRect rect;
+		m_pData->GetObjectBounds (m_pDoc, &rect);
+		m_pData->ShowSelection (false);
+		cairo_surface_t *surface = NULL;
+		if (!strcmp (type, "pdf"))
+			surface = cairo_pdf_surface_create_for_stream (cairo_write_func, output, w, h);
+		else {
+			surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, w, h);
+			if (!strcmp (type, "eps"))
+				cairo_ps_surface_set_eps (surface, TRUE);
+		}
+		cairo_t *cr = cairo_create (surface);
+		cairo_translate (cr, -rect.x0, -rect.y0);
+		cairo_surface_destroy (surface);
+		Render (cr);
+		cairo_destroy (cr);
 	} else if (!strcmp (type, "svg")) {
 		xmlDocPtr doc = BuildSVG ();
 		xmlIndentTreeOutput = true;
