@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * atom.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -30,6 +30,7 @@
 #include "xml-utils.h"
 #include <cmath>
 #include <cstring>
+#include <sstream>
 
 using namespace std;
 
@@ -95,7 +96,7 @@ void Atom::zoom (double ZoomFactor)
 	m_z *= ZoomFactor;
 }
 
-bool Atom::GetCoords (double *x, double *y, double *z)
+bool Atom::GetCoords (double *x, double *y, double *z) const
 {
 	if (!x || !y)
 		return false;
@@ -106,7 +107,7 @@ bool Atom::GetCoords (double *x, double *y, double *z)
 	return true;
 }
 
-const gchar* Atom::GetSymbol ()
+const gchar* Atom::GetSymbol () const
 {
 	Element* Elt = Element::GetElement(m_Z);
 	return (Elt)? Element::Symbol(m_Z): NULL;
@@ -154,7 +155,7 @@ void Atom::Transform2D (Matrix2D& m, double x, double y)
 	m_y += y;
 }
 
-xmlNodePtr Atom::Save (xmlDocPtr xml)
+xmlNodePtr Atom::Save (xmlDocPtr xml) const
 {
 	xmlNodePtr parent;
 	gchar buf[16];
@@ -212,17 +213,16 @@ bool Atom::LoadNode (xmlNodePtr node)
 	return true;
 }
 
-bool Atom::SaveNode (xmlDocPtr xml, xmlNodePtr node)
+bool Atom::SaveNode (xmlDocPtr xml, xmlNodePtr node) const
 {
 	return true;
 }
 
-Bond* Atom::GetBond (Atom* pAtom)
+Bond* Atom::GetBond (Atom* pAtom) const
 {
-	Bond* pBond = m_Bonds[pAtom];
-	if (!pBond)
-		m_Bonds.erase (pAtom);
-	return pBond;
+	std::map<Atom*, Bond*>::const_iterator i;
+	i = m_Bonds.find (pAtom);
+	return (i != m_Bonds.end ())? (*i).second: NULL;
 }
 
 bool Atom::SetProperty (unsigned property, char const *value)
@@ -256,7 +256,24 @@ bool Atom::SetProperty (unsigned property, char const *value)
 	}
 	return  true;
 }
-	
+
+string Atom::GetProperty (unsigned property) const
+{
+	ostringstream res;
+	switch (property) {
+	case GCU_PROP_POS2D: {
+		Document *doc = const_cast <Atom*> (this)->GetDocument ();
+		if (doc)
+			res << m_x / doc->GetScale () << " " << m_y / doc->GetScale ();
+		else
+			res << m_x << " " << m_y;
+	}
+	default:
+		break;
+	}
+	return res.str ();
+}
+
 bool Atom::IsInCycle (Cycle* pCycle)
 {
 	map<gcu::Atom*, gcu::Bond*>::iterator i, end = m_Bonds.end ();
@@ -266,4 +283,43 @@ bool Atom::IsInCycle (Cycle* pCycle)
 	return false;
 }
 
+bool Atom::Match (Atom *atom, AtomMatchState &state)
+{
+	if (m_Z != atom->m_Z)
+		return false;
+	if (m_Bonds.size () != atom->m_Bonds.size ())
+		return false;
+	unsigned n = state.mol1.size ();
+	// add the atoms to state
+	state.mol1[this] = n;
+	state.mol2[atom] = n;
+	state.atoms[n] = AtomPair (this, atom);
+	// compare bonded atoms
+	map<gcu::Atom*, gcu::Bond*>::iterator i, iend = m_Bonds.end ();
+	map<gcu::Atom*, gcu::Bond*>::iterator j, jend = atom->m_Bonds.end ();
+	for (i = m_Bonds.begin (); i != iend; i++) {
+		if (state.mol1.find ((*i).first) != state.mol1.end ())
+			continue; /* may be not enough: we might search which atom is
+		  				associated with this one and see if it is effectively
+			  			bonded to *atom */
+		for (j = atom->m_Bonds.begin (); j != jend; j++) {
+			if (state.mol2.find ((*j).first) != state.mol2.end ())
+				continue;
+			if ((*i).first->Match ((*j).first, state)) 
+				break;
+		}
+		if (j == jend)
+			break; // no matching atom has been fond;
+	}
+	if (i == iend)
+		return true;
+	// if we are there, something wrong happened, clean and return
+	unsigned max = state.mol1.size ();
+	for (unsigned i = n; i < max; i++) {
+		state.mol1.erase (state.atoms[n].atom1);
+		state.mol2.erase (state.atoms[n].atom2);
+	}
+	return false;
+}
+
 }	//	namespace gcu
