@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * formula.cc 
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,6 +26,8 @@
 #include "formula.h"
 #include "element.h"
 #include "residue.h"
+#include "document.h"
+#include "molecule.h"
 #include <glib/gi18n.h>
 #include <cctype>
 #include <cmath>
@@ -59,53 +61,6 @@ parse_error::what(int &start, int &length) const throw()
 	return m_msg.c_str();
 }
 	
-class FormulaElt
-{
-public:
-	FormulaElt ();
-	virtual ~FormulaElt ();
-	virtual string Markup ();
-	virtual void BuildRawFormula (map<int, int> &raw) = 0;
-	virtual int GetValence () = 0;
-	int stoich;
-};
-
-class FormulaAtom: public FormulaElt
-{
-public:
-	FormulaAtom (int Z);
-	virtual ~FormulaAtom ();
-	string Markup ();
-	void BuildRawFormula (map<int, int> &raw);
-	int GetValence ();
-	int elt;
-};
-
-class FormulaBlock: public FormulaElt
-{
-public:
-	FormulaBlock ();
-	virtual ~FormulaBlock ();
-	string Markup ();
-	void BuildRawFormula (map<int, int> &raw);
-	list<FormulaElt *> children;
-	int GetValence ();
-	int parenthesis;
-};
-
-class FormulaResidue: public FormulaElt
-{
-public:
-	FormulaResidue (Residue const *res, char const *symbol, int Z);
-	virtual ~FormulaResidue ();
-	string Markup ();
-	void BuildRawFormula (map<int, int> &raw);
-	int GetValence ();
-	Residue const *residue;
-	string Symbol;
-	GCU_RO_PROP (int, Z);
-};
-
 FormulaElt::FormulaElt ()
 {
 	stoich = 1;
@@ -243,18 +198,20 @@ int FormulaResidue::GetValence ()
 	return 1; // residues with other valences are not currently supported
 }
 
-static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
+bool Formula::AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous, int offset)
 {
 	if (*sz == 0)
 		return true;
-	int i = 0;
+	unsigned i = 0;
 	char sy[Residue::MaxSymbolLength + 1];
 	Residue const *r = NULL;
 	bool amb = ambiguous, local_amb;
 	if (*sz) {
 		// search for any abbreviation starting sz
 		strncpy (sy, sz, Residue::MaxSymbolLength);
-		i = Residue::MaxSymbolLength;
+		i = strlen (sz);
+		if (i > Residue::MaxSymbolLength)
+			i = Residue::MaxSymbolLength;
 		while (i > 0) {
 			sy[i] = 0;
 			r = Residue::GetResidue (sy, &local_amb);
@@ -263,9 +220,12 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 			i--;
 		}
 		if (r) {
-			result.push_back (new FormulaResidue (r, sy, (local_amb? Element::Z (sy): 0)));
+			FormulaResidue *fr = new FormulaResidue (r, sy, (local_amb? Element::Z (sy): 0));
+			fr->start = offset;
+			fr->end = offset + i - 1;
+			result.push_back (fr);
 			ambiguous |= local_amb;
-			if (AnalString (sz + i, result, ambiguous))
+			if (AnalString (sz + i, result, ambiguous, offset + i))
 				return true;
 			ambiguous = amb; // restore ambiguity state
 			delete result.back ();
@@ -274,11 +234,15 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 		if (islower (*sz)) {
 			/* we might have some abbreviation around there */
 		}
-		*sz = toupper (*sz);
+		if (!(m_ParseMode & GCU_FORMULA_PARSE_NO_CASE))
+			*sz = toupper (*sz);
 		if (strlen (sz) == 1) {
 			i = Element::Z (sz);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 1;
+				result.push_back (fa);
 				return true;
 			} else
 				return false;
@@ -288,30 +252,41 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 			sy [1] = 0;
 			i = Element::Z (sy);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 1, result, ambiguous))
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 1;
+				result.push_back (fa);
+				if (AnalString (sz + 1, result, ambiguous, offset + 1))
 					return true;
 				delete result.back ();
 				result.pop_back ();
 			}
-			sy[1] = tolower (sz[1]);
+			if (!(m_ParseMode & GCU_FORMULA_PARSE_NO_CASE))
+				sy[1] = tolower (sz[1]);
 			sy[2] = 0;
 			i = Element::Z (sy);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 2, result, ambiguous))
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 2;
+				result.push_back (fa);
+				if (AnalString (sz + 2, result, ambiguous, offset + 2))
 					return true;
 				delete result.back ();
 				result.pop_back ();
 			}
 			if (*sz != 'U')
 				return false;
-			sy[2] = tolower (sz[2]);
+			if (!(m_ParseMode & GCU_FORMULA_PARSE_NO_CASE))
+				sy[2] = tolower (sz[2]);
 			sy[3] = 0;
 			i = Element::Z (sy);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 3, result, ambiguous))
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 3;
+				result.push_back (fa);
+				if (AnalString (sz + 3, result, ambiguous, offset + 3))
 					return true;
 			}
 			return false;
@@ -320,12 +295,16 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 			sy[1] = sz[1];	
 			if (*sz == 'U') {
 				// No 2 chars symbols begining with U exist, so try 3 chars symbols
-				sy[2] = tolower (sz[2]);
+				if (!(m_ParseMode & GCU_FORMULA_PARSE_NO_CASE))
+					sy[2] = tolower (sz[2]);
 				sy[3] = 0;
 				i = Element::Z (sy);
 				if (i > 0) {
-					result.push_back (new FormulaAtom (i));
-					if (AnalString (sz + 3, result, ambiguous))
+					FormulaAtom *fa = new FormulaAtom (i);
+					fa->start = offset;
+					fa->end = offset + 3;
+					result.push_back (fa);
+					if (AnalString (sz + 3, result, ambiguous, offset + 3))
 						return true;
 					delete result.back ();
 					result.pop_back ();
@@ -334,8 +313,11 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 			sy[2] = 0;
 			i = Element::Z (sy);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 2, result, ambiguous))
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 2;
+				result.push_back (fa);
+				if (AnalString (sz + 2, result, ambiguous, offset + 2))
 					return true;
 				delete result.back ();
 				result.pop_back ();
@@ -343,8 +325,11 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 			sy[1] = 0;	
 			i = Element::Z (sy);
 			if (i > 0) {
-				result.push_back (new FormulaAtom (i));
-				if (AnalString (sz + 1, result, ambiguous))
+				FormulaAtom *fa = new FormulaAtom (i);
+				fa->start = offset;
+				fa->end = offset + 1;
+				result.push_back (fa);
+				if (AnalString (sz + 1, result, ambiguous, offset + 1))
 					return true;
 			}
 		}
@@ -352,9 +337,10 @@ static bool AnalString (char *sz, list<FormulaElt *> &result, bool &ambiguous)
 	return false;
 }
 
-Formula::Formula (string entry) throw (parse_error)
+Formula::Formula (string entry, FormulaParseMode mode) throw (parse_error)
 {
 	Entry = entry;
+	m_ParseMode = mode;
 	Parse (Entry, Details);
 	m_ConnectivityCached = m_WeightCached = false;
 }
@@ -474,6 +460,8 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 			string str (formula, i + 1, j - i - 2);
 			FormulaBlock *block = new FormulaBlock ();
 			block->parenthesis = k;
+			block->start = i;
+			block->end = j;
 			result.push_back (block);
 			try {
 				Parse (str, block->children);
@@ -494,7 +482,7 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 			sz = new char[k + 1];
 			strncpy (sz, formula.c_str () + i, k);
 			sz[k] = 0;
-			if (!AnalString (sz, result, ambiguous)) {
+			if (!AnalString (sz, result, ambiguous, i)) {
 				delete [] sz;
 				throw parse_error (_("Could not interpret the symbol list"), i, k);
 			}
@@ -513,21 +501,20 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 			throw parse_error (_("Invalid character"), i, 1);
 	}
 	if (ambiguous) {
-		int replaced = 1, max = 0;
-		// first count ambiguous symbols
-		list<FormulaElt *>::iterator it, end = result.end ();
-		for (it = result.begin (); it != end; it++) {
-			if (dynamic_cast <FormulaResidue *> (*it) != NULL)
-				max++;
+		switch (m_ParseMode & 7) {
+		case GCU_FORMULA_PARSE_GUESS: {
+			// if it fails, nothing is replaced 
+			if (!TryReplace (result, result.begin ()))
+				g_warning ("ambiguous formula");
+			break;
 		}
-		if (!BuildConnectivity ()) {
-			// for now just replace all ambiguous residues
-			it = result.begin ();
+		case GCU_FORMULA_PARSE_ATOM: {
+			// replace all ambiguous residues
+			list<FormulaElt *>::iterator it = result.begin (), end = result.end ();
 			FormulaResidue *res;
 			while (it != result.end ()) {
 				res = dynamic_cast <FormulaResidue *> (*it);
 				if (res && res->GetZ ()) {
-					printf("found ambiguous with Z=%d\n",res->GetZ());
 					FormulaAtom *elt = new FormulaAtom (res->GetZ());
 					elt->stoich =  res->stoich;
 					it = result.erase (it);
@@ -536,6 +523,15 @@ void Formula::Parse (string &formula, list<FormulaElt *> &result) throw (parse_e
 				} else
 					it++;
 			}
+			break;
+		}
+		case GCU_FORMULA_PARSE_RESIDUE:
+			// don't do anything
+			break;
+		case GCU_FORMULA_PARSE_ASK: {
+			// FIXME: really ask
+			break;
+		}
 		}
 	}
 }
@@ -567,11 +563,16 @@ void Formula::CalculateIsotopicPattern (IsotopicPattern &pattern)
 	i = Raw.begin ();
 	if (i == end) // empty formula
 		return;
-	IsotopicPattern *pat, *pat0;
-	pat = Element::GetElement ((*i).first)->GetIsotopicPattern ((*i).second);
+	IsotopicPattern *pat = NULL, *pat0;
+	while (!pat && (i != end)) {
+		pat = Element::GetElement ((*i).first)->GetIsotopicPattern ((*i).second);
+		i++;
+	}
+	if (!pat)
+		return;
 	pattern.Copy (*pat);
 	pat->Unref ();
-	for (i++; i != end; i++) {
+	for (; i != end; i++) {
 		pat = Element::GetElement ((*i).first)->GetIsotopicPattern ((*i).second);
 		if (!pat) {
 			// no stable isotope known for the element
@@ -589,8 +590,45 @@ void Formula::CalculateIsotopicPattern (IsotopicPattern &pattern)
 
 bool Formula::BuildConnectivity ()
 {
-	// FIXME: write this function
-	return false;
+	Document *Doc = new Document (NULL);
+	Molecule *mol = Molecule::MoleculeFromFormula (Doc, *this, false);
+	bool result = mol;
+	delete Doc;
+	return result;
+}
+
+bool Formula::TryReplace (list<FormulaElt *> &result, list<FormulaElt *>::iterator it)
+{
+	if (BuildConnectivity ())
+		return true;
+	FormulaResidue *res;
+	while (it != result.end ()) {
+		res = dynamic_cast <FormulaResidue *> (*it);
+		if (res && res->GetZ ())
+			break;
+		it++;
+	}
+	if (it == result.end ())
+		return false;
+	list<FormulaElt *>::iterator next = it;
+	next++;
+	if (TryReplace (result, next))
+		return true;
+	FormulaAtom *elt = new FormulaAtom (res->GetZ());
+	elt->stoich =  res->stoich;
+	it = result.erase (it);
+	result.insert (it, elt);
+	next = it;
+	next++;
+	bool ret = TryReplace (result, next);
+	if (!ret) {
+		it--;
+		delete (*it);
+		it = result.erase (it);
+		result.insert (it, res);
+	} else
+		delete res;
+	return ret;
 }
 
 }	//	namespace gcu
