@@ -24,9 +24,13 @@
 
 #include "config.h"
 #include <gcu/application.h>
+#include <gcu/atom.h>
 #include <gcu/document.h>
+#include <gcu/formula.h>
 #include <gcu/loader.h>
+#include <gcu/molecule.h>
 #include <gcu/objprops.h>
+#include <gcu/residue.h>
 
 #include <goffice/app/module-plugin-defs.h>
 #include <openbabel/chemdrawcdx.h>
@@ -35,20 +39,22 @@
 #include <vector>
 #include <cstdio>
 #include <libintl.h>
+#include <cstring>
 
 using namespace std;
 using namespace gcu;
 
 #if G_BYTE_ORDER == G_LITTLE_ENDIAN
-#define READINT16(input,buf,i) gsf_input_read (input, 2, (guint8*) &i)
-#define READINT32(input,buf,i) gsf_input_read (input, 4, (guint8*) &i)
+#define READINT16(input,i) gsf_input_read (input, 2, (guint8*) &i)
+#define READINT32(input,i) gsf_input_read (input, 4, (guint8*) &i)
 #else
-#define READINT16(input,buf,i) \
-	bool res = gsf_input_read (input, 2, (guint8*) buf), \
-	(guint16) i = buf[0] + buf[1] << 8, res
+char *buffer[4];
+#define READINT16(input,i) \
+	bool res = gsf_input_read (input, 2, (guint8*) buffer), \
+	(guint16) i = buffer[0] + buffer[1] << 8, res
 #define READINT32(input,buf,i) \
-	bool res = gsf_input_read (input, 4, (guint8*) buf), \
-	(guint32) i = buf[0] + buf[1] << 8 + buf[2] << 16 + buf[3] << 24, res
+	bool res = gsf_input_read (input, 4, (guint8*) buffer), \
+	(guint32) i = buffer[0] + buffer[1] << 8 + buffer[2] << 16 + buffer[3] << 24, res
 #endif
 
 typedef struct {
@@ -79,7 +85,6 @@ private:
 	bool ReadGroup (GsfInput *in, Object *parent);
 	bool ReadGraphic (GsfInput *in, Object *parent);
 	bool ReadFragmentText (GsfInput *in, Object *parent);
-	bool ReadResidue (GsfInput *in, Object *parent);
 	guint16 ReadSize (GsfInput *in);
 	bool ReadDate (GsfInput *in);
 
@@ -102,6 +107,8 @@ CDXLoader::~CDXLoader ()
 
 bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOContext *io)
 {
+	if (doc == NULL || in == NULL)
+		return false;
 	bool result = true;
 	guint16 code;
 	bufsize = 64;
@@ -110,7 +117,7 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 	if (!gsf_input_read (in, kCDX_HeaderLength, (guint8*) buf) || strncmp (buf, kCDX_HeaderString, kCDX_HeaderStringLen)) {
 		result = false;
 		code = 0;
-	} else if (!READINT16 (in, buf, code)) {
+	} else if (!READINT16 (in, code)) {
 		result = false;
 		code = 0;
 	}
@@ -166,7 +173,7 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 					break;
 				}
 				guint32 length;
-				if (!READINT32 (in, buf, length)) {
+				if (!READINT32 (in,length)) {
 					result = false;
 					break;
 				}
@@ -177,13 +184,13 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 			case kCDXProp_FontTable: {
 				// skip origin platform and read fonts number
 				guint16 nb;
-				if (gsf_input_seek (in, 2, G_SEEK_CUR) || !READINT16 (in, buf, nb))
+				if (gsf_input_seek (in, 2, G_SEEK_CUR) || !READINT16 (in,nb))
 					return false;
 				CDXFont font;
 				for (int i = 0; i < nb; i++) {
-					if (!READINT16 (in, buf, font.index) ||
-						!READINT16 (in, buf, font.encoding) ||
-						!READINT16 (in, buf, size))
+					if (!READINT16 (in,font.index) ||
+						!READINT16 (in,font.encoding) ||
+						!READINT16 (in,size))
 						return false;
 					gsf_input_read (in, size, (guint8*) buf);
 					buf[size] = 0;
@@ -196,11 +203,11 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 				colors.push_back ("1 1 1"); // white
 				colors.push_back ("0 0 0"); // black
 				unsigned nb = (size - 2) / 6;
-				if (!READINT16 (in, buf, size) || size != nb)
+				if (!READINT16 (in,size) || size != nb)
 					return false;
 				guint16 red, blue, green;
 				for (unsigned i = 0; i < nb; i++) {
-				if (!READINT16 (in, buf, red) || !READINT16 (in, buf, green) || !READINT16 (in, buf, blue))
+				if (!READINT16 (in,red) || !READINT16 (in,green) || !READINT16 (in,blue))
 					return false;
 					snprintf (buf, bufsize, "%g %g %g", (double) red / 0xffff, (double) green / 0xffff, (double) blue / 0xffff);
 					colors.push_back (buf);
@@ -214,7 +221,7 @@ bool CDXLoader::Read  (Document *doc, GsfInput *in, char const *mime_type, IOCon
 		}
 		if (!result)
 			break;
-		if (!READINT16 (in, buf, code)) {
+		if (!READINT16 (in,code)) {
 			result = false;
 			break;
 		}
@@ -242,7 +249,7 @@ bool CDXLoader::ReadGenericObject  (GsfInput *in)
 	guint16 code;
 	if (gsf_input_seek (in, 4, G_SEEK_CUR)) //skip the id
 		return false;
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -255,7 +262,7 @@ bool CDXLoader::ReadGenericObject  (GsfInput *in)
 			if (size && !gsf_input_read (in, size, (guint8*) buf))
 				return false;
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	return true;
@@ -266,7 +273,7 @@ bool CDXLoader::ReadPage (GsfInput *in, Object *parent)
 	guint16 code;
 	if (gsf_input_seek (in, 4, G_SEEK_CUR)) //skip the id
 		return false;
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -298,7 +305,7 @@ bool CDXLoader::ReadPage (GsfInput *in, Object *parent)
 			if (size && !gsf_input_read (in, size, (guint8*) buf))
 				return false;
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	return true;
@@ -307,23 +314,23 @@ bool CDXLoader::ReadPage (GsfInput *in, Object *parent)
 bool CDXLoader::ReadMolecule (GsfInput *in, Object *parent)
 {
 	guint16 code;
-	Object *Molecule = Object::CreateObject ("molecule", parent);
+	Object *mol = Object::CreateObject ("molecule", parent);
 	guint32 Id;
-	if (!READINT32 (in, buf, Id))
+	if (!READINT32 (in,Id))
 		return false;
 	snprintf (buf, bufsize, "m%d", Id);
-	Molecule->SetId (buf);
-	if (!READINT16 (in, buf, code))
+	mol->SetId (buf);
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
 			switch (code) {
 			case kCDXObj_Node:
-				if (!ReadAtom (in, Molecule))
+				if (!ReadAtom (in, mol))
 					return false;
 				break;
 			case kCDXObj_Bond:
-				if (!ReadBond (in, Molecule))
+				if (!ReadBond (in, mol))
 					return false;
 				break;
 			default:
@@ -337,9 +344,10 @@ bool CDXLoader::ReadMolecule (GsfInput *in, Object *parent)
 			if (size && !gsf_input_read (in, size, (guint8*) buf))
 				return false;
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
+		static_cast <Molecule*> (mol)->UpdateCycles ();
 	return true;
 }
 
@@ -350,34 +358,155 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 	Document *Doc = NULL;
 	Atom->SetProperty (GCU_PROP_ATOM_Z, "6");
 	guint32 Id;
+	int type = 0;
 	int Z = 6;
-	if (!READINT32 (in, buf, Id))
+	if (!READINT32 (in,Id))
 		return false;
 	snprintf (buf, bufsize, "a%d", Id);
 	Atom->SetId (buf);
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
 			switch (code) {
 			case kCDXObj_Fragment: {
-				Doc = parent->GetDocument ()->GetApp ()->CreateNewDocument ();
+				if (!Doc)
+					Doc = parent->GetDocument ()->GetApp ()->CreateNewDocument ();
 				Doc->SetProperty (GCU_PROP_THEME_BOND_LENGTH, "943718");
 				if (!ReadMolecule (in, Doc)) {
 					delete Doc;
 					return false;
 				}
-/*				static int i = 0;
-				char *filename = g_strdup_printf("file:///home/jean/devel/samples/%d.gchempaint", i++);
-				Doc->SetProperty (GCU_PROP_DOC_FILENAME, filename);
-				g_free (filename);
-				Doc->Save ();*/
 				break;
 			}
 			case kCDXObj_Text:
 				if (Z == 6) {
 					if (!ReadFragmentText (in, Atom))
 						goto bad_exit;
+				switch (type) {
+				case 0:
+					// Parse the formula.
+					try {
+						Formula form (buf, GCU_FORMULA_PARSE_RESIDUE);
+						if (Doc) {
+							map< string, Object * >::iterator i;
+							Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+							if (Doc->GetChildrenNumber () != 1 || mol == NULL)
+								goto bad_exit;
+							// compare the formula as interpreted with the document contents
+							// TODO: write this code
+						}
+						// now build a molecule from the formula
+						Molecule *mol2 = NULL;
+						if (Doc)
+							mol2 = Molecule::MoleculeFromFormula (Doc, form);
+						bool replace = true;
+						if (mol2) {
+						} else {
+							// check if the formula contains only one atom
+							std::list<FormulaElt *> const &items = form.GetElements ();
+							if (items.size () == 1 && dynamic_cast <FormulaAtom const *> (items.front ()))
+								replace = false;
+						}
+						if (replace) {
+							string pos = Atom->GetProperty (GCU_PROP_POS2D);
+							Molecule *mol = dynamic_cast <Molecule *> (parent);
+							if (mol)
+								mol->Remove (Atom);
+							delete Atom;
+							Atom = Object::CreateObject ("fragment", parent);
+							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+							snprintf (buf, bufsize, "a%d", Id);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+						}
+						if (!Doc)
+							Doc = parent->GetDocument ()->GetApp ()->CreateNewDocument ();
+					}
+					catch (parse_error &error) {
+						return false;
+					}
+					break;
+				case 4: {
+					bool amb;
+					Residue const *res = Residue::GetResidue (buf, &amb);
+					if (res != NULL) {
+						map< string, Object * >::iterator i;
+						Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+						if (mol == NULL)
+							goto bad_exit;
+						if (*res == *mol) {
+							// Residue has been identified to the known one
+							string pos = Atom->GetProperty (GCU_PROP_POS2D);
+							Molecule *mol = dynamic_cast <Molecule *> (parent);
+							if (mol)
+								mol->Remove (Atom);
+							delete Atom;
+							Atom = Object::CreateObject ("fragment", parent);
+							Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+							snprintf (buf, bufsize, "a%d", Id);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+							Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+							Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+						} else {
+							// FIXME: should the document care with the residues?
+						}
+					} else {
+						// FIXME: Unkown residue: add it to the database? or just to the document?
+					}
+					break;
+				}
+				case 5:
+					// First, parse the formula.
+					{
+						map< string, Object * >::iterator i;
+						Molecule *mol = dynamic_cast <Molecule *> (Doc->GetFirstChild (i));
+						// Do the molecule have a pseudo-atom?
+						bool have_pseudo = false;
+						Object *obj = mol->GetFirstChild (i);
+						gcu::Atom *a;
+						while (obj) {
+							a = dynamic_cast <gcu::Atom *> (obj);
+							if (a && ! a->GetZ ()) {
+								have_pseudo = true;
+								break;
+							}
+							obj = mol->GetNextChild (i);
+						}
+						if (mol == NULL)
+							goto bad_exit;
+						try {
+							// First, parse the formula.
+							Formula form (buf, GCU_FORMULA_PARSE_RESIDUE);
+							// now build a molecule from the formula
+							Molecule *mol2 = Molecule::MoleculeFromFormula (Doc, form, have_pseudo);
+							// Now see if it matches with the molecule
+							if (!mol2 || !(*mol == *mol2)) {
+								// try adding a new residue
+								printf("failed for %s\n",buf);
+							}
+						}
+						catch (parse_error &error) {
+							int start, length;
+							puts (error.what (start, length));
+						}
+						string pos = Atom->GetProperty (GCU_PROP_POS2D);
+						mol = dynamic_cast <Molecule *> (parent);
+						if (mol)
+							mol->Remove (Atom);
+						delete Atom;
+						Atom = Object::CreateObject ("fragment", parent);
+						Atom->SetProperty (GCU_PROP_TEXT_TEXT, buf);
+						snprintf (buf, bufsize, "a%d", Id);
+						Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_ID, buf);
+						Atom->SetProperty (GCU_PROP_FRAGMENT_ATOM_START, "0");
+						Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
+					}
+					break;
+				default:
+					break;
+				}
 					break;
 				}
 			default:
@@ -391,14 +520,14 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 			switch (code) {
 			case kCDXProp_2DPosition: {
 				gint32 x, y;
-				if (size != 8 || !READINT32 (in, buf, y) || !READINT32 (in, buf, x))
+				if (size != 8 || !READINT32 (in,y) || !READINT32 (in,x))
 					goto bad_exit;
 				snprintf (buf, bufsize, "%d %d", x, y);
 				Atom->SetProperty (GCU_PROP_POS2D, buf);
 				break;
 			}
 			case kCDXProp_Node_Element:
-				if (size != 2 || !READINT16 (in, buf, size))
+				if (size != 2 || !READINT16 (in,size))
 					goto bad_exit;
 				Z = size;
 				snprintf (buf, bufsize, "%u", size);
@@ -412,17 +541,19 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 				Atom->SetProperty (GCU_PROP_ATOM_CHARGE, buf);
 				break;
 			case kCDXProp_Node_Type:
-				if (size != 2 || !READINT16 (in, buf, size))
+				if (size != 2 || !READINT16 (in,type))
 					goto bad_exit;
-				switch (size) {
-				case 4:	// nickname
-				case 5:	// fragment
-					// We use the same representation as a fragment
-					// First: convert the atom to a fragment
-					break;
-				case 12:	// attachement point
+				if (type == 12) {
 					// convert the atom to a pseudo atom.
-					break;
+					string pos = Atom->GetProperty (GCU_PROP_POS2D);
+					Molecule *mol = dynamic_cast <Molecule *> (parent);
+					if (mol)
+						mol->Remove (Atom);
+					delete Atom;
+					Atom = Object::CreateObject ("pseudo-atom", parent);
+					snprintf (buf, bufsize, "a%d", Id);
+					Atom->SetId (buf);
+					Atom->SetProperty (GCU_PROP_POS2D, pos.c_str ());
 				}
 				break;
 			default:
@@ -430,7 +561,7 @@ bool CDXLoader::ReadAtom (GsfInput *in, Object *parent)
 					goto bad_exit;
 			}
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			goto bad_exit;
 	}
 	if (Doc)
@@ -447,12 +578,12 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 	guint16 code;
 	Object *Bond = Object::CreateObject ("bond", parent);
 	guint32 Id;
-	if (!READINT32 (in, buf, Id))
+	if (!READINT32 (in,Id))
 		return false;
 	snprintf (buf, bufsize, "b%d", Id);
 	Bond->SetId (buf);
 	Bond->SetProperty (GCU_PROP_BOND_ORDER, "1");
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -464,21 +595,21 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 				return false;
 			switch (code) {
 			case kCDXProp_Bond_Begin: {
-				if (size != 4 || !READINT32 (in, buf, Id))
+				if (size != 4 || !READINT32 (in,Id))
 					return false;
 				snprintf (buf, bufsize, "%u", Id);
 				Bond->SetProperty (GCU_PROP_BOND_BEGIN, buf);
 				break;
 			}
 			case kCDXProp_Bond_End: {
-				if (size != 4 || !READINT32 (in, buf, Id))
+				if (size != 4 || !READINT32 (in,Id))
 					return false;
 				snprintf (buf, bufsize, "%u", Id);
 				Bond->SetProperty (GCU_PROP_BOND_END, buf);
 				break;
 			}
 			case kCDXProp_Bond_Order:
-				if (size != 2 || !READINT16 (in, buf, size))
+				if (size != 2 || !READINT16 (in,size))
 					return false;
 				switch (size) {
 				case 2:
@@ -493,7 +624,7 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 				}
 				break;
 			case kCDXProp_Bond_Display:
-				if (size != 2 || !READINT16 (in, buf, size))
+				if (size != 2 || !READINT16 (in,size))
 					return false;
 				switch (size) {
 				case 1:
@@ -525,7 +656,7 @@ bool CDXLoader::ReadBond (GsfInput *in, Object *parent)
 					return false;
 			}
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	return true;
@@ -536,11 +667,11 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 	guint16 code;
 	Object *Text= Object::CreateObject ("text", parent);
 	guint32 Id;
-	if (!READINT32 (in, buf, Id))
+	if (!READINT32 (in,Id))
 		return false;
 	snprintf (buf, bufsize, "t%d", Id);
 	Text->SetId (buf);
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -555,9 +686,9 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 				if (size != 8)
 					return false;
 				gint32 x, y;
-				if (!READINT32 (in, buf, y))
+				if (!READINT32 (in,y))
 					return false;
-				if (!READINT32 (in, buf, x))
+				if (!READINT32 (in,x))
 					return false;
 				snprintf (buf, bufsize, "%d %d", x, y);
 				Text->SetProperty (GCU_PROP_POS2D, buf);
@@ -565,7 +696,7 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 			}
 			case kCDXProp_Text: {
 				guint16 nb;
-				if (!READINT16 (in, buf, nb))
+				if (!READINT16 (in,nb))
 					return false;
 				size -=2;
 				for (int i =0; i < nb; i++) {
@@ -573,7 +704,7 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 						return false;
 					guint16 n[5];
 					for (int j = 0; j < 5; j++)
-						if (!READINT16 (in, buf, n[j]))
+						if (!READINT16 (in,n[j]))
 							return false;
 					size -= 10;
 				}
@@ -590,7 +721,7 @@ bool CDXLoader::ReadText (GsfInput *in, Object *parent)
 					return false;
 			}
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	return true;
@@ -603,7 +734,7 @@ bool CDXLoader::ReadGroup (GsfInput *in, Object *parent)
 	Group->Lock ();
 	if (gsf_input_seek (in, 4, G_SEEK_CUR)) //skip the id
 		return false;
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -627,7 +758,7 @@ bool CDXLoader::ReadGroup (GsfInput *in, Object *parent)
 			if (size && !gsf_input_read (in, size, (guint8*) buf))
 				return false;
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	Group->Lock (false);
@@ -641,7 +772,7 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 	guint32 Id;
 	guint16 type = 0xffff, arrow_type = 0xffff;
 	gint32 x0, y0, x1, y1;
-	if (!READINT32 (in, buf, Id) || !READINT16 (in, buf, code))
+	if (!READINT32 (in,Id) || !READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -653,16 +784,16 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 				return false;
 			switch (code) {
 			case kCDXProp_BoundingBox:
-				if (size != 16 || !READINT32 (in, buf, y1) || !READINT32 (in, buf, x1)
-					|| !READINT32 (in, buf, y0) || !READINT32 (in, buf, x0))
+				if (size != 16 || !READINT32 (in,y1) || !READINT32 (in,x1)
+					|| !READINT32 (in,y0) || !READINT32 (in,x0))
 					return false;
 				break;
 			case kCDXProp_Graphic_Type:
-				if (size != 2 || !READINT16 (in, buf, type))
+				if (size != 2 || !READINT16 (in,type))
 					return false;
 				break;
 			case kCDXProp_Arrow_Type:
-				if (size != 2 || !READINT16 (in, buf, arrow_type))
+				if (size != 2 || !READINT16 (in,arrow_type))
 					return false;
 				break;
 			default:
@@ -670,7 +801,7 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 					return false;
 			}
 		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	if (type == 1) {
@@ -709,7 +840,7 @@ bool CDXLoader::ReadGraphic  (GsfInput *in, Object *parent)
 guint16 CDXLoader::ReadSize  (GsfInput *in)
 {
 	guint16 size;
-	if (!READINT16 (in, buf, size))
+	if (!READINT16 (in,size))
 		return 0xffff;
 	if ((unsigned) size + 1 > bufsize) {
 		do
@@ -725,7 +856,7 @@ bool CDXLoader::ReadDate  (GsfInput *in)
 {
 	guint16 n[7];
 	for (int i = 0; i < 7; i++)
-		if (!READINT16 (in, buf, n[i]))
+		if (!READINT16 (in,n[i]))
 			return false;
 	GDate *date = g_date_new_dmy (n[2], (GDateMonth) n[1], n[0]);
 	g_date_strftime (buf, bufsize, "%m/%d/%Y", date);
@@ -738,7 +869,7 @@ bool CDXLoader::ReadFragmentText (GsfInput *in, Object *parent)
 	guint16 code;
 	if (gsf_input_seek (in, 4, G_SEEK_CUR)) //skip the id
 		return false;
-	if (!READINT16 (in, buf, code))
+	if (!READINT16 (in,code))
 		return false;
 	while (code) {
 		if (code & kCDXTag_Object) {
@@ -749,21 +880,21 @@ bool CDXLoader::ReadFragmentText (GsfInput *in, Object *parent)
 			if ((size = ReadSize (in)) == 0xffff)
 				return false;
 			switch (code) {
-			case kCDXProp_2DPosition: {
+/*			case kCDXProp_2DPosition: {
 				if (size != 8)
 					return false;
 				gint32 x, y;
-				if (!READINT32 (in, buf, y))
+				if (!READINT32 (in, y))
 					return false;
-				if (!READINT32 (in, buf, x))
+				if (!READINT32 (in, x))
 					return false;
 				snprintf (buf, bufsize, "%d %d", x, y);
 //				Text->SetProperty (GCU_PROP_POS2D, buf);
 				break;
-			}
+			}*/
 			case kCDXProp_Text: {
 				guint16 nb;
-				if (!READINT16 (in, buf, nb))
+				if (!READINT16 (in,nb))
 					return false;
 				size -=2;
 				for (int i =0; i < nb; i++) {
@@ -771,7 +902,7 @@ bool CDXLoader::ReadFragmentText (GsfInput *in, Object *parent)
 						return false;
 					guint16 n[5];
 					for (int j = 0; j < 5; j++)
-						if (!READINT16 (in, buf, n[j]))
+						if (!READINT16 (in,n[j]))
 							return false;
 					size -= 10;
 				}
@@ -780,41 +911,14 @@ bool CDXLoader::ReadFragmentText (GsfInput *in, Object *parent)
 				if (!gsf_input_read (in, size, (guint8*) buf))
 					return false;
 				buf[size] = 0;
-//				Text->SetProperty (GCU_PROP_TEXT_TEXT, buf);
 				break;
 			}
 			default:
-				if (size && !gsf_input_read (in, size, (guint8*) buf))
+				if (size && gsf_input_seek (in, size, G_SEEK_CUR))
 					return false;
 			}
 		}
-		if (!READINT16 (in, buf, code))
-			return false;
-	}
-	return true;
-}
-
-bool CDXLoader::ReadResidue (GsfInput *in, Object *parent)
-{
-	guint16 code;
-	if (gsf_input_seek (in, 4, G_SEEK_CUR)) //skip the id
-		return false;
-	if (!READINT16 (in, buf, code))
-		return false;
-	while (code) {
-		if (code & kCDXTag_Object) {
-printf("residue child = %x\n",code);
-			if (!ReadGenericObject (in))
-				return false;
-		} else {
-printf("residue property = %x\n",code);
-			guint16 size;
-			if ((size = ReadSize (in)) == 0xffff)
-				return false;
-			if (size && !gsf_input_read (in, size, (guint8*) buf))
-				return false;
-		}
-		if (!READINT16 (in, buf, code))
+		if (!READINT16 (in,code))
 			return false;
 	}
 	return true;
