@@ -34,6 +34,8 @@
 #include <gcu/chemistry.h>
 #include <goffice/goffice.h>
 #include <goffice/utils/go-file.h>
+#include <cstring>
+#include <cstdio>
 
 void cb_print_version (const gchar *option_name, const gchar *value, gpointer data, GError **error)
 {
@@ -50,10 +52,35 @@ static GOptionEntry options[] =
 {
 	{ "version", 'v', G_OPTION_FLAG_NO_ARG, G_OPTION_ARG_CALLBACK, (void*) cb_print_version, N_("Prints GChem3d Viewer version"), NULL },
 	{ "bgcolor", 'b', 0, G_OPTION_ARG_STRING, &bgcolor, N_("Background color: white, black or #rrggbb (default is black)"), NULL },
-	{ "display3d", 'd', 0, G_OPTION_ARG_STRING, &display3d, N_("How molecules are displayed; possible values are BallnStick (the default) and SpaceFill"), NULL },
+	{ "display3d", 'd', 0, G_OPTION_ARG_STRING, &display3d, N_("How molecules are displayed; possible values are BallnStick (the default), SpaceFill, Cylinders, and Wireframe"), NULL },
 	{ NULL }
 };
 
+static char const *Display3DModeNames[] = {
+	"ball&stick",
+	"spacefill",
+	"cylinders",
+	"wireframe"
+};
+
+static Display3DMode display3d_mode_from_string (char const *mode)
+{
+	if (mode == NULL)
+		return BALL_AND_STICK;
+	// first ensure the string is in lower case
+	char lcmode[16];
+	int i, max = strlen (mode), res = WIREFRAME;
+	if (max > 15)
+		return BALL_AND_STICK;
+	for (i = 0; i < max; i++)
+		lcmode[i] = tolower (mode[i]);
+	lcmode[i] = 0;
+	while (res >= BALL_AND_STICK && strcmp (lcmode, Display3DModeNames[res]))
+		res--;
+	return (Display3DMode) res;
+}
+
+
 int main(int argc, char *argv[])
 {
 	GnomeVFSURI *uri, *auri;
@@ -68,8 +95,6 @@ int main(int argc, char *argv[])
 		printf ("Could not initialize GnomeVFS\n");
 		return 1;
 	}
-	/* Initialize libgoffice */
-	libgoffice_init ();
 
 	if (argc > 1 && argv[1][0] == '-') {
 		context = g_option_context_new (_(" [file]"));
@@ -84,7 +109,7 @@ int main(int argc, char *argv[])
 		}
 	}
 
-	gc3dApplication *App = new gc3dApplication ();
+	gc3dApplication *App = new gc3dApplication (display3d_mode_from_string (display3d), bgcolor);
 	gc3dDocument *pDoc = App->OnFileNew();
 	path = g_get_current_dir ();
 	dir = g_strconcat (path, "/", NULL);
