@@ -2,7 +2,7 @@
 
 /* 
  * Gnome Chemistry Utils
- * programs/gchemcalc.cc 
+ * programs/calc/gchemcalc.cc 
  *
  * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
  *
@@ -27,8 +27,17 @@
 #undef PACKAGE
 #define PACKAGE "gchemutils-unstable" 
 #include <gcu/application.h>
+#include <gcu/atom.h>
+#include <gcu/bond.h>
+#include <gcu/document.h>
 #include <gcu/element.h>
+#include <gcu/filechooser.h>
 #include <gcu/formula.h>
+#include <gcu/molecule.h>
+#include <gcu/printable.h>
+#include <gcu/print-setup-dlg.h>
+#include <gcu/residue.h>
+#include <gcu/value.h>
 #include <glib/gi18n.h>
 #include <glade/glade.h>
 #include <gtk/gtkmain.h>
@@ -61,31 +70,193 @@
 #include <goffice/utils/go-marker.h>
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-output-memory.h>
+#include <gsf/gsf-output-gio.h>
 #include <libgnomevfs/gnome-vfs.h>
-#include <cmath>
+#include <libxml/tree.h>
 #include <iostream>
+#include <cmath>
+#include <cstring>
 
 using namespace gcu;
 
 using namespace std;
 
-class GChemCalc: public Application
+class GChemCalc: public Application, public Printable
 {
 public:
 	GChemCalc ();
 	Formula formula;
 	GtkLabel *markup, *raw, *weight, *mono, *monomass;
+	GtkWidget *graph_widget;
 	GtkWidget *pattern_page;
+	GtkWindow *window;
+	GogGraph *graph;
 	GogChart *chart;
 	GogLabel *label;
 	GogPlot *plot;
 	GogSeries *series;
 	GtkListStore *pclist;
+
+	GtkWindow *GetGtkWindow () {return window;}
+	void DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const;
+	void OnSaveAsImage ();
+	bool FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc = NULL);
+
+private:
+	void ParseNodes (xmlNodePtr node);
+GCU_PROP (GtkUIManager *, UIManager)
 };
 
-GChemCalc::GChemCalc (): Application ("gchemcalc-unstable"),
-formula ("")
+// need a way to create atoms, bonds, and molecules for ambiguous symbols evaluation
+static Object* CreateAtom ()
 {
+	return new Atom ();
+}
+
+static Object* CreateBond ()
+{
+	return new Bond ();
+}
+
+static Object* CreateMolecule ()
+{
+	return new Molecule ();
+}
+
+static Object* CreateDocument ()
+{
+	return new Document ();
+}
+
+GChemCalc::GChemCalc ():
+	Application ("gchemcalc-unstable"),
+	formula ("")
+{
+	Object::AddType ("atom", CreateAtom, AtomType);
+	Object::AddType ("pseudo-atom", CreateAtom);
+	Object::AddType ("bond", CreateBond, BondType);
+	Object::AddType ("molecule", CreateMolecule, MoleculeType);
+	Object::AddType ("document", CreateDocument, DocumentType);
+	// Load residues
+	xmlDocPtr doc;
+	char *name;
+	doc = xmlParseFile (PKGDATADIR"/residues.xml");
+	if (doc) {
+		if (!strcmp ((char*) doc->children->name, "residues"))
+			ParseNodes (doc->children->children);
+		xmlFreeDoc (doc);
+	}
+	name = g_strconcat (getenv ("HOME"), "/.gchemutils/residues.xml", NULL);
+	if (g_file_test (name, G_FILE_TEST_EXISTS) && (doc = xmlParseFile (name))) {
+		if (!strcmp ((char*) doc->children->name, "residues"))
+			ParseNodes (doc->children->children);
+		xmlFreeDoc (doc);
+	}
+	g_free (name);
+}
+
+void GChemCalc::ParseNodes (xmlNodePtr node)
+{
+	Residue* r;
+	while (node) {
+		if (!strcmp ((char*) node->name, "residue")) {
+			r = new Residue ();
+			r->Load (node);
+		}
+		node = node->next;
+	}
+}
+
+void GChemCalc::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
+{
+	cairo_t *cr;
+	gdouble width, height;
+
+	cr = gtk_print_context_get_cairo_context (context);
+	width = gtk_print_context_get_width (context);
+	height = gtk_print_context_get_height (context);
+
+	int w, h; // size in points
+	w = graph_widget->allocation.width;
+	h = graph_widget->allocation.height;
+	switch (GetScaleType ()) {
+	case GCU_PRINT_SCALE_NONE:
+		break;
+	case GCU_PRINT_SCALE_FIXED:
+		w *= Printable::GetScale ();
+		h *= Printable::GetScale ();
+		break;
+	case GCU_PRINT_SCALE_AUTO:
+		if (GetHorizFit ())
+			w = width;
+		if (GetVertFit ())
+			h = height;
+		break;
+	}
+	double x = 0., y = 0.;
+	if (GetHorizCentered ())
+		x = (width - w) / 2.;
+	if (GetVertCentered ())
+		y = (height - h) / 2.;
+	cairo_save (cr);
+	cairo_translate (cr, x, y);
+	gog_graph_render_to_cairo (graph, cr, w, h);
+	cairo_restore (cr);
+}
+
+void GChemCalc::OnSaveAsImage ()
+{
+	list<string> l;
+	char const *mime;
+	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
+	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
+		l.push_front ((*i).first.c_str ());
+	if (go_image_get_format_from_name ("eps") != GO_IMAGE_FORMAT_UNKNOWN) {
+		mime = go_image_format_to_mime ("eps");
+		if (mime)
+			l.push_front (mime);
+	}
+	l.push_front ("application/postscript");
+	l.push_front ("application/pdf");
+	l.push_front ("image/svg+xml");
+	FileChooser (this, true, l, NULL, _("Save as image"), GetImageSizeWidget ());
+}
+
+bool GChemCalc::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
+{
+	if(bSave) {
+		GFile *file = g_file_new_for_uri (filename);
+		bool err = g_file_query_exists (file, NULL);
+		gint result = GTK_RESPONSE_YES;
+		if (err) {
+			gchar * message = g_strdup_printf (_("File %s\nexists, overwrite?"), filename);
+			GtkDialog* Box = GTK_DIALOG (gtk_message_dialog_new (NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, message));
+			gtk_window_set_icon_name (GTK_WINDOW (Box), "gspectrum");
+			result = gtk_dialog_run (Box);
+			gtk_widget_destroy (GTK_WIDGET (Box));
+			g_free (message);
+		}
+		if (result == GTK_RESPONSE_YES) {
+			g_file_delete (file, NULL, NULL);
+			char *fname = go_mime_to_image_format (mime_type);
+			GOImageFormat format = go_image_get_format_from_name ((fname)? fname: filename);
+			if (format == GO_IMAGE_FORMAT_UNKNOWN)
+				return true;
+			GError *error = NULL;
+			GsfOutput *output = gsf_output_gio_new_for_uri (filename, &error);
+			if (error) {
+				g_error_free (error);
+				return true;
+			}
+			GogGraph *gr = gog_graph_dup (graph);
+			gog_graph_set_size (gr, GetImageWidth (), GetImageHeight ());
+			gog_graph_export_image (gr, format, output, -1., -1.);
+			g_object_unref (gr);
+			
+		}
+		g_object_unref (file);
+	}
+	return false;
 }
 
 GChemCalc *App;
@@ -172,23 +343,13 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		format = g_strconcat (_("Raw formula:"), " \t", App->formula.GetRawMarkup (), NULL);
 		gtk_label_set_markup (App->raw, format);
 		g_free (format);
-		int prec;
 		bool artificial;
-		double weight = App->formula.GetMolecularWeight (prec, artificial);
-		if (prec > 0) {
-			format = g_strdup_printf ("%%0.%df",prec);
-		} else {
-			if (prec < 0) {
-				// round the value to replace not significant figures by 0s.
-				double offs = pow10 (prec);
-				weight = rint (weight * offs) / offs;
-			}
-			format = artificial? g_strdup ("(%.0f)"): g_strdup ("%.0f");
-		}
-		char *weightstr = g_strdup_printf (format, weight);
-		gtk_label_set_text (App->weight, weightstr);
+		DimensionalValue weight = App->formula.GetMolecularWeight (artificial);
+		char *weightstr = (artificial)?
+			g_strdup_printf ("(%.0f g.mol<sup>-1</sup>)",weight.GetAsDouble ()):
+			g_strdup (weight.GetAsString ());			
+		gtk_label_set_markup (App->weight, weightstr);
 		g_free (weightstr);
-		g_free (format);
 		// Composition
 		gtk_list_store_clear (App->pclist);
 		map<int,int> &raw = App->formula.GetRawFormula ();
@@ -214,7 +375,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		Element *elt;
 		if (nC > 0) {
 			elt = Element::GetElement (6);
-			pcent = nC * elt->GetWeight (prec) / weight * 100.;
+			pcent = nC * elt->GetWeight ()->GetAsDouble () / weight.GetAsDouble () * 100.;
 			weightstr = g_strdup_printf ((artificial)? "(%.0f)": "%.2f", pcent);
 			gtk_list_store_append (App->pclist, &iter);
 			gtk_list_store_set (App->pclist, &iter,
@@ -225,7 +386,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		}
 		if (nH > 0) {
 			elt = Element::GetElement (1);
-			pcent = nH * elt->GetWeight (prec) / weight * 100.;
+			pcent = nH * elt->GetWeight ()->GetAsDouble () / weight.GetAsDouble () * 100.;
 			weightstr = g_strdup_printf ((artificial)? "(%.0f)": "%.2f", pcent);
 			gtk_list_store_append (App->pclist, &iter);
 			gtk_list_store_set (App->pclist, &iter,
@@ -238,7 +399,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 		for (k = elts.begin (); k != kend; k++) {
 			nC = (*k).second;
 			elt = Element::GetElement ((*k).first.c_str ());
-			pcent = nC * elt->GetWeight (prec) / weight * 100.;
+			pcent = nC * elt->GetWeight ()->GetAsDouble () / weight.GetAsDouble () * 100.;
 			weightstr = g_strdup_printf ((artificial)? "(%.0f)": "%.2f", pcent);
 			gtk_list_store_append (App->pclist, &iter);
 			gtk_list_store_set (App->pclist, &iter,
@@ -258,9 +419,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 			gtk_widget_hide (App->pattern_page);
 			return;
 		} else {
-			weightstr = g_strdup_printf ("%g", pattern.GetMonoMass ());
-			gtk_label_set_text (App->monomass, weightstr);
-			g_free (weightstr);
+			gtk_label_set_text (App->monomass, pattern.GetMonoMass ().GetAsString ());
 			gtk_widget_show (App->pattern_page);
 			nb = pattern.GetValues (&values);
 			// correct mean mass (for high molecular weights)
@@ -270,7 +429,7 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 				t += pcent;
 				m += i * pcent;
 			}
-			mass = (int) rint (weight - m / t);
+			mass = (int) rint (weight.GetAsDouble () - m / t);
 			// do not display values < 0.1
 			min = 0;
 			while (values[min] < 0.1)
@@ -324,7 +483,9 @@ static void cb_entry_active (GtkEntry *entry, gpointer data)
 	}
 }
 
-static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, GogGraph *graph)
+static GogGraph *graph = NULL;
+
+static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_data,  guint info, gpointer data)
 {
 	guchar *buffer = NULL;
 	char *format = NULL;
@@ -385,9 +546,10 @@ static void on_get_data (GtkClipboard *clipboard, GtkSelectionData *selection_da
 	}
 }
 
-void on_clear_data(GtkClipboard *clipboard, GogGraph *graph)
+void on_clear_data(GtkClipboard *clipboard, gpointer data)
 {
 	g_object_unref (graph);
+	graph = NULL;
 }
 
 static GtkTargetEntry const targets[] = {
@@ -397,18 +559,67 @@ static GtkTargetEntry const targets[] = {
 	{(char *) "image/png", 0, 3}
 };
 
-static void on_copy (GOGraphWidget *widget)
+static void on_copy ()
 {
 	GtkClipboard* clipboard = gtk_clipboard_get (GDK_SELECTION_CLIPBOARD);
+	if (graph)
+		g_object_unref (graph);
+	graph = (GogGraph*) gog_object_dup (GOG_OBJECT (go_graph_widget_get_graph (GO_GRAPH_WIDGET (App->graph_widget))), NULL, NULL);
 	gtk_clipboard_set_with_data (clipboard, targets, 4,
 		(GtkClipboardGetFunc) on_get_data, (GtkClipboardClearFunc) on_clear_data,
-		gog_object_dup (GOG_OBJECT (go_graph_widget_get_graph (widget)), NULL, NULL));
+		NULL);
+}
+
+static void on_print ()
+{
+	App->Print (false);
+}
+
+static void on_print_preview ()
+{
+	App->Print (true);
+}
+
+static void on_page_setup ()
+{
+	new PrintSetupDlg (App, App);
+}
+
+static void on_save_as_image ()
+{
+	App->OnSaveAsImage ();
+}
+
+static void on_mode (GtkRadioAction *action, GtkRadioAction *current)
+{
+	App->formula.SetParseMode (static_cast <FormulaParseMode> (gtk_radio_action_get_current_value (action)));
+}
+
+static void on_page (GtkNotebook *book, GtkNotebookPage *p, int page)
+{
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/SaveAsImage"), page);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/PageSetup"), page);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/PrintPreview"), page);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/FileMenu/Print"), page);
+	gtk_widget_set_sensitive (gtk_ui_manager_get_widget (App->GetUIManager (), "/MainMenu/EditMenu/Copy"), page);
 }
 
 static GtkActionEntry entries[] = {
   { "FileMenu", NULL, N_("_File") },
+	  { "SaveAsImage", GTK_STOCK_SAVE_AS, N_("Save As _Image..."), "<control>I",
+		  N_("Save the current file as an image"), G_CALLBACK (on_save_as_image) },
+	  { "PageSetup", NULL, N_("Page Set_up..."), NULL,
+		  N_("Setup the page settings for your current printer"), G_CALLBACK (on_page_setup) },
+	  { "PrintPreview", GTK_STOCK_PRINT_PREVIEW, N_("Print Pre_view"), NULL,
+		  N_("Print preview"), G_CALLBACK (on_print_preview) },
+	  { "Print", GTK_STOCK_PRINT, N_("_Print..."), "<control>P",
+		  N_("Print the current file"), G_CALLBACK (on_print) },
 	  { "Quit", GTK_STOCK_QUIT, N_("_Quit"), "<control>Q",
 		  N_("Quit GChemCalc"), G_CALLBACK (on_quit) },
+  { "EditMenu", NULL, N_("_Edit") },
+	  { "Copy", GTK_STOCK_COPY, N_("_Copy"), "<control>C",
+		  N_("Copy the selection"), G_CALLBACK (on_copy) },
+  { "ModeMenu", NULL, N_("_Mode") },
   { "HelpMenu", NULL, N_("_Help") },
 	  { "Help", GTK_STOCK_HELP, N_("_Contents"), "F1",
 		  N_("View help for the Chemical Calculator"), G_CALLBACK (on_help) },
@@ -422,12 +633,42 @@ static GtkActionEntry entries[] = {
 		  N_("About GChemCalc"), G_CALLBACK (on_about) }
 };
 
+static GtkRadioActionEntry radios[] = {
+	{ "Guess", NULL, N_("_Guess"), NULL,
+		N_("Try to guess what is correct when interpreting ambiguous symbols"),
+		GCU_FORMULA_PARSE_GUESS },
+	{ "Atom", NULL, N_("_Atom"), NULL,
+		N_("Interpreting ambiguous symbols as atoms"),
+		GCU_FORMULA_PARSE_ATOM },
+	{ "Residue", NULL, N_("_Nickname"), NULL,
+		N_("Interpret ambiguous symbols as atoms groups nicknames"),
+		GCU_FORMULA_PARSE_RESIDUE },
+	{ "Ask", NULL, N_("As_k"), NULL,
+		N_("Ask user for the correct interpretation of ambiguous symbols"),
+		GCU_FORMULA_PARSE_ASK },
+};
+
 static const char *ui_description =
 "<ui>"
 "  <menubar name='MainMenu'>"
 "    <menu action='FileMenu'>"
+"      <menuitem action='SaveAsImage'/>"
+"	   <separator name='file-sep1'/>"
+"      <menuitem action='PageSetup'/>"
+"      <menuitem action='PrintPreview'/>"
+"      <menuitem action='Print'/>"
+"	   <separator name='file-sep2'/>"
 "      <menuitem action='Quit'/>"
 "    </menu>"
+"    <menu action='EditMenu'>"
+"      <menuitem action='Copy'/>"
+"    </menu>"
+"    <menu action='ModeMenu'>"
+"      <menuitem action='Guess'/>"
+"      <menuitem action='Atom'/>"
+"      <menuitem action='Residue'/>"
+"      <menuitem action='Ask'/>"
+"    </menu>"
 "    <menu action='HelpMenu'>"
 "      <menuitem action='Help'/>"
 "      <placeholder name='mail'/>"
@@ -506,28 +747,28 @@ int main (int argc, char *argv[])
 		cout << _("For usage see: gchemcalc-unstable [-?|--help]") << endl;
 		return -1;
 	}
+
+	App = new GChemCalc ();
 	
-	/* Initialize libgoffice */
-	libgoffice_init ();
 	/* Initialize plugins manager */
 	go_plugins_init (NULL, NULL, NULL, NULL, TRUE, GO_PLUGIN_LOADER_MODULE_TYPE);
 
-	App = new GChemCalc ();
-
 	GladeXML *xml =  glade_xml_new (GLADEDIR"/gchemcalc.glade", "gchemcalc", NULL);
-	GtkWidget *window = glade_xml_get_widget (xml, "gchemcalc");
-	g_signal_connect (GTK_OBJECT (window), "destroy",
+	App->window = GTK_WINDOW (glade_xml_get_widget (xml, "gchemcalc"));
+	g_signal_connect (GTK_OBJECT (App->window), "destroy",
 		 G_CALLBACK (gtk_main_quit),
 		 NULL);
 	
 	GtkWidget *vbox = glade_xml_get_widget (xml, "vbox1");
 	GtkUIManager *ui_manager = gtk_ui_manager_new ();
+	App->SetUIManager (ui_manager);
 	GtkActionGroup *action_group = gtk_action_group_new ("MenuActions");
 	gtk_action_group_set_translation_domain (action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions (action_group, entries, G_N_ELEMENTS (entries), NULL);
+	gtk_action_group_add_radio_actions (action_group, radios, G_N_ELEMENTS (radios), 0, G_CALLBACK (on_mode), NULL);
 	gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
 	GtkAccelGroup *accel_group = gtk_ui_manager_get_accel_group (ui_manager);
-	gtk_window_add_accel_group (GTK_WINDOW (window), accel_group);
+	gtk_window_add_accel_group (GTK_WINDOW (App->window), accel_group);
 	if (!gtk_ui_manager_add_ui_from_string (ui_manager, ui_description, -1, &error)) {
 		g_message ("building menus failed: %s", error->message);
 		g_error_free (error);
@@ -577,13 +818,14 @@ int main (int argc, char *argv[])
 	App->monomass = GTK_LABEL (glade_xml_get_widget (xml, "monomass"));
 	App->pattern_page = glade_xml_get_widget (xml, "pattern");
 #ifdef GO_GRAPH_WIDGET_OLD_API
-	GtkWidget *pw = go_graph_widget_new ();
+	App->graph_widget = go_graph_widget_new ();
 #else
-	GtkWidget *pw = go_graph_widget_new (NULL);
+	App->graph_widget = go_graph_widget_new (NULL);
 #endif
-	gtk_widget_show (pw);
-	gtk_box_pack_end (GTK_BOX (App->pattern_page), pw, TRUE, TRUE, 0);
-	App->chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (pw));
+	gtk_widget_show (App->graph_widget);
+	gtk_box_pack_end (GTK_BOX (App->pattern_page), App->graph_widget, TRUE, TRUE, 0);
+	App->graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (App->graph_widget));
+	App->chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (App->graph_widget));
 	App->plot = (GogPlot *) gog_plot_new_by_name ("GogXYPlot");
 	gog_object_add_by_name (GOG_OBJECT (App->chart), "Plot", GOG_OBJECT (App->plot));
 	// Create a series for the plot and populate it with some simple data
@@ -603,17 +845,21 @@ int main (int argc, char *argv[])
 	GtkWidget *w = glade_xml_get_widget (xml, "entry");
 	g_signal_connect (GTK_OBJECT (w), "activate",
 		 G_CALLBACK (cb_entry_active),
-		 window);
+		 App->window);
 	gcu_element_load_databases ("isotopes", NULL);
+	Element::LoadBODR ();
 	if (argc == 1){
 		gtk_entry_set_text (GTK_ENTRY (w), argv[0]);
-		cb_entry_active (GTK_ENTRY (w), window);
+		cb_entry_active (GTK_ENTRY (w), App->window);
 	}
 
-	w = glade_xml_get_widget (xml, "copy");
-	g_signal_connect_swapped (w, "clicked", G_CALLBACK (on_copy), pw);
+	w = glade_xml_get_widget (xml, "notebook1");
+	g_signal_connect (w, "switch-page", G_CALLBACK (on_page), NULL);
+	on_page (NULL, NULL, 0); // force menus deactivation
 
 	gtk_main ();
+	if (graph)
+		g_object_unref (graph);
 	delete App;
 	return 0;
 }
