@@ -23,20 +23,24 @@
  */
 
 #include "config.h"
-#include <gtk/gtk.h>
-#include <glade/glade.h>
-#include <libgnomeprint/gnome-print.h>
-#include <libgnomeprint/gnome-print-job.h>
-#include <libgnomeprintui/gnome-print-dialog.h>
-#include <libgnomeprintui/gnome-print-job-preview.h>
-#include <libgnomevfs/gnome-vfs-ops.h>
-#include <libgnomevfs/gnome-vfs-utils.h>
 #include "application.h"
 #include "globals.h"
-#include <gcu/filechooser.h>
 #include "prefs.h"
 #include "window.h"
+#include <gcu/filechooser.h>
+#include <goffice/utils/go-image.h>
+#include <gsf/gsf-output-gio.h>
+#include <glade/glade.h>
+#include <libgnomevfs/gnome-vfs-ops.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
+#include <gtk/gtk.h>
+#include <cairo-pdf.h>
+#include <cairo-ps.h>
 #include <glib/gi18n.h>
+#include <cstring>
+
+using namespace gcu;
+using namespace std;
 
 static unsigned short nNewDocs = 1;
 
@@ -94,48 +98,6 @@ bool gcApplication::OnFileClose ()
 	return true;
 }
 
-void gcApplication::OnFilePrint ()
-{
-	GnomePrintConfig* config = gnome_print_config_default ();
-	GnomePrintContext *pc;
-	GnomePrintJob *gpj = gnome_print_job_new (config);
-	int do_preview = 0, copies = 1, collate = 0;
-	GnomePrintDialog *gpd;
-	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new (gpj, (const guchar*) "Print test", GNOME_PRINT_DIALOG_COPIES));
-	gtk_window_set_icon_name (GTK_WINDOW (gpd), "gcrystal");
-	gnome_print_dialog_set_copies (gpd, copies, collate);
-	switch (gtk_dialog_run (GTK_DIALOG (gpd))) {
-	case GNOME_PRINT_DIALOG_RESPONSE_PRINT:
-		do_preview = 0;
-		break;
-	case GNOME_PRINT_DIALOG_RESPONSE_PREVIEW:
-		do_preview = 1;
-		break;
-	case GNOME_PRINT_DIALOG_RESPONSE_CANCEL:
-		gtk_widget_destroy (GTK_WIDGET (gpd));
-		return;
-	}
-	gtk_widget_destroy (GTK_WIDGET (gpd));
-	pc = gnome_print_job_get_context (gpj);
-	gnome_print_beginpage (pc, (const guchar*) "");
-	gdouble width, height;
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_WIDTH, &width);
-	gnome_print_config_get_double (config, (const guchar*) GNOME_PRINT_KEY_PAPER_HEIGHT, &height);
-	m_pActiveDoc->GetActiveView ()->Print (pc, width, height);
-	gnome_print_showpage (pc);
-	g_object_unref (pc);
-	gnome_print_job_close (gpj);
-	if (do_preview) {
-		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*)_("Preview"));
-		gtk_window_set_icon_name (GTK_WINDOW (preview), "gcrystal");
-		gtk_widget_show (preview);
-	} else {
-		gnome_print_job_print (gpj);
-	}
-	g_object_unref (gpj);
-	gnome_print_config_unref (config);
-}
-
 void gcApplication::OnSaveAsImage ()
 {
 	if (!m_pActiveDoc)
@@ -144,6 +106,9 @@ void gcApplication::OnSaveAsImage ()
 	map<string, GdkPixbufFormat*>::iterator i, end = m_SupportedPixbufFormats.end ();
 	for (i = m_SupportedPixbufFormats.begin (); i != end; i++)
 		l.push_front ((*i).first.c_str ());
+	l.push_front ("image/x-eps");
+	l.push_front ("application/postscript");
+	l.push_front ("application/pdf");
 	l.push_front ("model/vrml");
 	FileChooser (this, true, l, m_pActiveDoc, _("Save as image"), GetImageSizeWidget ());
 }
@@ -178,9 +143,22 @@ enum {
 	GCRYSTAL,
 	CIF,
 	VRML,
+	PDF,
+	PS,
+	EPS,
 	PIXBUF
 };
 
+static cairo_status_t cairo_write_func (void *closure, const unsigned char *data, unsigned int length)
+{
+	gboolean result;
+	GsfOutput *output = GSF_OUTPUT (closure);
+
+	result = gsf_output_write (output, length, data);
+
+	return result ? CAIRO_STATUS_SUCCESS : CAIRO_STATUS_WRITE_ERROR;
+}
+
 bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *pDoc)
 {
 	gcDocument *Doc = static_cast<gcDocument*> (pDoc);
@@ -194,6 +172,12 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			type = CIF;
 		else if (!strcmp (mime_type, "model/vrml"))
 			type = VRML;
+		else if (!strcmp (mime_type, "image/x-eps"))
+			type = EPS;
+		else if (!strcmp (mime_type, "application/postscript"))
+			type = PS;
+		else if (!strcmp (mime_type, "application/pdf"))
+			type = PDF;
 		else if ((pixbuf_type = GetPixbufTypeName (filename2, mime_type)))
 			type = PIXBUF;
 		char const *ext = NULL;
@@ -207,6 +191,15 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 		case VRML:
 			ext = ".wrl";
 			break;
+		case PDF:
+			ext = ".pdf";
+			break;
+		case PS:
+			ext = ".ps";
+			break;
+		case EPS:
+			ext = ".eps";
+			break;
 		default:
 			break;
 		}
@@ -254,6 +247,41 @@ bool gcApplication::FileProcess (const gchar* filename, const gchar* mime_type,
 			case PIXBUF:
 				Doc->SaveAsImage (filename2, pixbuf_type, options);
 				break;
+			default: {
+				char *fnm = go_mime_to_image_format (mime_type);
+				GOImageFormat format = go_image_get_format_from_name (fnm);
+				GError *error = NULL;
+				GsfOutput *output = gsf_output_gio_new_for_uri (filename2.c_str (), &error);
+				if (error) {
+					gchar * mess = g_strdup_printf (_("Could not create stream!\n%s"), error->message);
+					GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, mess);
+					g_free (mess);
+					gtk_dialog_run (GTK_DIALOG (message));
+					gtk_widget_destroy (message);
+					g_error_free (error);
+					return true;
+				}
+				cairo_surface_t *surface = NULL;
+				switch (format) {
+					case GO_IMAGE_FORMAT_EPS:
+						surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						cairo_ps_surface_set_eps (surface, TRUE);
+						break;
+					case GO_IMAGE_FORMAT_PDF:
+						surface = cairo_pdf_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						break;
+					case GO_IMAGE_FORMAT_PS:
+						surface = cairo_ps_surface_create_for_stream (cairo_write_func, output, GetImageWidth (), GetImageHeight ());
+						break;
+					default:
+						return true;
+				}
+				cairo_t *cr = cairo_create (surface);
+				cairo_surface_destroy (surface);
+				Doc->GetView ()->RenderToCairo (cr, GetImageWidth (), GetImageHeight ());
+				cairo_destroy (cr);
+				break;
+			}
 			}
 	} else {
 		if (!strcmp (mime_type, "application/x-gcrystal"));
