@@ -4,7 +4,7 @@
  * Gnome Chemistry Utils
  * programs/gchemtable-curve.cc 
  *
- * Copyright (C) 2005-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2005-2008 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,6 +24,7 @@
 
 #include "config.h"
 #include "gchemtable-curve.h"
+#include "gchemtable-data.h"
 #include "gchemtable-data-allocator.h"
 #include <gcu/chemistry.h>
 #include <gcu/element.h>
@@ -292,9 +293,9 @@ static const char *ui_web_description =
 
 GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 	Dialog (App, GLADEDIR"/curve.glade", "curvedlg"),
-	Printable ()
+	Printable (),
+	m_Guru (NULL)
 {
-	m_Name = name;
 	m_GraphBox = glade_xml_get_widget (xml, "vbox1");
 	GtkUIManager *ui_manager = gtk_ui_manager_new ();
 	GtkActionGroup *action_group = gtk_action_group_new ("MenuActions");
@@ -323,9 +324,15 @@ GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 	gtk_widget_set_size_request (m_GraphWidget, 400, 250);
 	gtk_widget_show (m_GraphWidget);
 	gtk_box_pack_end (GTK_BOX (m_GraphBox), m_GraphWidget, TRUE, TRUE, 0);
+	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
 	GogChart *chart = go_graph_widget_get_chart (GO_GRAPH_WIDGET (m_GraphWidget));
+	if (!name) {
+		OnProperties ();
+		return;
+	}
 	GogPlot *plot = (GogPlot *) gog_plot_new_by_name ("GogXYPlot");
 	gog_object_add_by_name (GOG_OBJECT (chart), "Plot", GOG_OBJECT (plot));
+	m_Name = name;
 	// Create a series for the plot and populate it with some simple data
 	GogSeries *series = gog_plot_new_series (plot);
 	double *yvals = g_new0 (double, MAX_ELT);
@@ -504,21 +511,23 @@ GChemTableCurve::GChemTableCurve (GChemTableApp *App, char const *name):
 	gog_dataset_set_dim (GOG_DATASET (obj), GOG_AXIS_ELEM_MAX, data, &error);
 	data = go_data_vector_val_new (yvals, MAX_ELT, g_free);
 	gog_series_set_dim (series, 1, data, &error);
+	gog_series_set_dim (series, 0, gct_data_vector_get_from_name (_("Atomic number")), &error);
 	obj = gog_object_get_child_by_role (GOG_OBJECT (chart),
 			gog_object_find_role_by_name (GOG_OBJECT (chart), "X-Axis"));
 	data = go_data_scalar_str_new ("Z", FALSE);
 	label = (GogObject*) g_object_new (GOG_LABEL_TYPE, NULL);
 	gog_dataset_set_dim (GOG_DATASET (label), 0, data, &error);
 	gog_object_add_by_name (obj, "Label", label);
-	m_Graph = go_graph_widget_get_graph (GO_GRAPH_WIDGET (m_GraphWidget));
 }
 
 GChemTableCurve::~GChemTableCurve ()
 {
+	if (m_Guru)
+		gtk_widget_destroy (m_Guru);
 	curves.erase (m_Name);
 }
 
-void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *context)
+void GChemTableCurve::DoPrint (GtkPrintOperation *print, GtkPrintContext *context) const
 {
 	cairo_t *cr;
 	gdouble width, height;
@@ -572,6 +581,8 @@ static void
 graph_user_config_free_data (gpointer data,
 					  GClosure *closure)
 {
+	GChemTableCurve *curve = gct_control_gui_get_owner (GCT_CONTROL_GUI (data));
+	curve->ClearGuru ();
 	g_object_unref (data);
 	closure->data = NULL;
 }
@@ -591,9 +602,9 @@ void GChemTableCurve::OnProperties ()
 	gct_control_gui_set_owner (tcg, this);
 	GClosure *closure = g_cclosure_new (G_CALLBACK (on_update_graph), tcg,
 					(GClosureNotify) graph_user_config_free_data);
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), 
-								  GTK_WINDOW (gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg),
-		      											NULL, closure)));
+	m_Guru = gog_guru (m_Graph, GOG_DATA_ALLOCATOR (tcg), NULL, closure);
+	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (m_Guru));
+	gtk_widget_show (m_Guru);
 	g_closure_sink (closure);
 }
 
